local MAJOR, MINOR = "libCandyIcon-2.0", 1
local libCandyIcon, oldminor = LibStub:NewLibrary(MAJOR, MINOR)
if not libCandyIcon then return end

local ANIM_OFF = 0
local ANIM_FADEIN = 1
local ANIM_FADEOUT = 2
local ANIM_IDLE = 3


function post_shrink(self)
	self.Parent:SetWidth (self.Parent.lci_width)
	self.Parent:SetHeight (self.Parent.lci_height)
end


function pre_shrink(self)
	self.Parent:SetWidth (self.Parent.lci_width * 2)
	self.Parent:SetHeight (self.Parent.lci_height * 2)
end


local function CandyIconFinishedFunc(self)
	s = self.Parent.lci_state

	if s == ANIM_FADEIN then
		self.Parent.lci_state = ANIM_IDLE
		self.Parent.lci_ao_idle:Play()
	elseif s == ANIM_IDLE then
	elseif s == ANIM_FADEOUT then
		self.Parent.lci_state = ANIM_OFF
		self.Parent:lci_Hide()
	elseif s == ANIM_OFF then
	end
end


--
-- functions added to the candy icon frame object
--
local function CandyIconPlayFunction(self)
	self:lci_Show()
	self.lci_state = ANIM_FADEIN

	if self.lci_ao_out:IsPlaying() then self.lci_ao_out:Stop() end
	if self.lci_ao_idle:IsPlaying() then self.lci_ao_idle:Stop() end

	self:SetAlpha(0)

	self.lci_ao_in:Play()
end


local function CandyIconStopFunction(self)
	self.lci_state = ANIM_FADEOUT

	if self.lci_ao_in:IsPlaying() then self.lci_ao_in:Stop() end
	if self.lci_ao_idle:IsPlaying() then self.lci_ao_idle:Stop() end

	self:SetAlpha(self.lci_alpha)

	self.lci_ao_out:Play()
end


function pre_idle(self)
	self.Parent:SetAlpha (self.Parent.lci_alpha)
end


function MakeAnimations(f, anim, anim_str, duration, smoothing)
	if string.find (anim_str, "shrink") ~= nil then

		-- thought the "alpha trick" would work with this too,
		-- but it doesn't. but if we don't scale (by leaving
		-- out the middle part), changing the size works fine.

		-- using a self-made animation (with OnUpdate an "manual"
		-- scaling) won't do the trick either since the size
		-- is reset after the animation is finished but after
		-- OnFinished is fired.

		-- enlarge the frame before we can shrink it to its real size.
		local ao = anim:CreateAnimation ("Animation")
		ao:SetDuration (0)
		ao:SetOrder(1)
		ao.Parent = f
		ao:SetScript("OnUpdate", pre_shrink)

		local ao = anim:CreateAnimation ("Scale")
		ao:SetDuration (duration)
		ao:SetScale(0.5, 0.5)
		ao:SetSmoothing(smoothing)
		ao:SetOrder(2)

		-- and reset the real size.
		local ao = anim:CreateAnimation ("Animation")
		ao:SetDuration (0)
		ao:SetOrder(3)
		ao.Parent = f
		ao:SetScript("OnUpdate", post_shrink)
	end

	if string.find (anim_str, "magnify") ~= nil then
		local ao = anim:CreateAnimation ("Scale")
		ao:SetDuration (duration)
		ao:SetScale(2.0, 2.0)
		ao:SetSmoothing(smoothing)
		ao:SetOrder(2)
	end

	if string.find (anim_str, "bounce") ~= nil then
		local ao = anim:CreateAnimation ("Scale")
		ao:SetDuration (duration)
		ao:SetScale(0.5, 0.5)
		ao:SetSmoothing(smoothing)
		ao:SetOrder(2)
	end

	if string.find (anim_str, "screw") ~= nil then
		local ao = anim:CreateAnimation ("Scale")
		ao:SetDuration (duration)
		ao:SetScale(0.5, 0.5)
		ao:SetSmoothing(smoothing)
		ao:SetOrder(2)

		local ao = anim:CreateAnimation ("Rotation")
		ao:SetDuration (duration)
		ao:SetDegrees (360)
		ao:SetSmoothing(smoothing)
		ao:SetOrder(2)
	end


	if string.find (anim_str, "id") ~= nil then
		local ao = anim:CreateAnimation ("Scale")
		ao:SetDuration (duration)
		ao:SetScale(1.0, 1.0)
		ao:SetSmoothing(smoothing)
		ao:SetOrder(2)
	end
end


--
-- the visible API
--

function libCandyIcon:MakeCandyIcon(f, values)
	f.lci_alpha = f:GetAlpha()
	f.lci_height = f:GetHeight()
	f.lci_width = f:GetWidth()

	f.lci_state = ANIM_OFF

	fadein_anim = values["fadein_anim"] or ""
	fadeout_anim = values["fadeout_anim"] or ""
	idle_anim = values["idle_anim"] or ""

	fadein_duration = values["fadein_duration"] or 0.5
	fadeout_duration = values["fadeout_duration"] or 0.5
	idle_duration = values["idle_duration"] or 0.5

	f.Play = CandyIconPlayFunction
	f.Stop = CandyIconStopFunction

	f.lci_Hide = f.Hide
	f.lci_Show = f.Show


	--
	-- fade in
	--
	local anim = f:CreateAnimationGroup()
	anim:SetScript("OnFinished", CandyIconFinishedFunc)
	anim.Parent = f
	f.lci_ao_in = anim

	local ao = anim:CreateAnimation ("Alpha", "fi_alpha")
	ao:SetDuration (fadein_duration)
	ao:SetChange(1)
	ao:SetSmoothing("IN")
	ao:SetOrder(2)

	MakeAnimations(f, anim, fadein_anim, fadein_duration, "IN")

	anim:SetLooping("NONE")


	--
	-- idle
	--
	local anim = f:CreateAnimationGroup()
	anim:SetScript("OnFinished", CandyIconFinishedFunc)
	anim.Parent = f
	f.lci_ao_idle = anim

	-- dirty workaround for fade in alpha always reseting to 0.
	-- if the idle animation changes the alpha, this magicaly
	-- stops working (well, as magicaly as this keeps the frame
	-- from disapearing).
	local ao = anim:CreateAnimation ("Scale", "idle_foo")
	ao.Parent = f
	ao:SetDuration (0)	-- causes this animation to fire only twice.
	ao:SetOrder(1)
	ao:SetScript("OnUpdate", pre_idle)

	MakeAnimations(f, anim, idle_anim, idle_duration, "IN_OUT")
	anim:SetLooping("BOUNCE")

	--
	-- fade out
	--
	local anim = f:CreateAnimationGroup()
	anim.Parent = f
	anim:SetScript("OnFinished", CandyIconFinishedFunc)
	f.lci_ao_out = anim

	local ao = anim:CreateAnimation ("Alpha", "fo_alpha")
	ao:SetDuration (fadeout_duration)
	ao:SetChange(-1)
	ao:SetSmoothing("OUT")
	ao:SetOrder(2)
	MakeAnimations(f, anim, fadeout_anim, fadeout_duration, "OUT")

	anim:SetLooping("NONE")
end

