--[[
	Project.: Broker: ButtonFacade
	File....: Core.lua
	Version.: 45
	Author..: StormFX
]]

local BF = LibStub("AceAddon-3.0"):GetAddon("ButtonFacade")

-- [ Libraries ] --

local LDB = LibStub("LibDataBroker-1.1")
local Icon = LibStub("LibDBIcon-1.0", true)
local LBF = LibStub("LibButtonFacade")

-- [ Set Up ] --

local BBF = LibStub("AceAddon-3.0"):NewAddon("Broker_ButtonFacade", "AceConsole-3.0")

-- [ Localization ] --

-- Hard-code enUS/enGB.
local L = {
	["Click to open the options window."] = "Click to open the options window.",
	["Current Skin:"] = "Current Skin:",
}
-- Automatically inject all other locales. Please use the localization application on WoWAce.com to update these.
-- http://www.wowace.com/projects/buttonfacade/localization/namespaces/buttontest/
do
	local LOC = GetLocale()
	if LOC == "deDE" then
L["Click to open the options window."] = "Klicken Sie zum Öffnen des Optionen-Fenster."
L["Current Skin:"] = "Aktuelle Haut:"

	elseif LOC == "esES" then
L["Click to open the options window."] = "Haga clic para abrir la ventana de opciones."
L["Current Skin:"] = "Actual Textura:"

	elseif LOC == "esMX" then
L["Click to open the options window."] = "Haga clic para abrir la ventana de opciones."
L["Current Skin:"] = "Actual Textura:"

	elseif LOC == "frFR" then
L["Click to open the options window."] = "Clic gauche pour ouvrir la fenêtre des options."
L["Current Skin:"] = "Actuel de la peau:"

	elseif LOC == "koKR" then
L["Click to open the options window."] = "클릭하면 설정창이 열립니다."
L["Current Skin:"] = "현재 스킨 :"

	elseif LOC == "ruRU" then
L["Click to open the options window."] = "Нажмите, чтобы открыть окно параметров."
L["Current Skin:"] = "Текущие Кожа:"

	elseif LOC == "zhCN" then
L["Click to open the options window."] = "单击以打开选项窗口。"
L["Current Skin:"] = "当前皮肤："

	elseif LOC == "zhTW" then
L["Click to open the options window."] = "左擊打開選項視窗。"
L["Current Skin:"] = "當前皮膚："

	end
end

-- [ Local Variables ] --

local db

-- [ Local Functions ] --

-- Toggles the minimap icon.
local function ToggleIcon()
	if Icon then
		db.hide = not db.hide
		if db.hide then
			Icon:Hide(BBF.name)
		else
			Icon:Show(BBF.name)
		end
	end
end

-- [ Core Methods ] --

-- Initialize the add-on.
function BBF:OnInitialize()
	-- Validate the DB.
	if not BrokerBFDB then
		BrokerBFDB = {}

		-- Defaults
		BrokerBFDB = {
			hide = true,
			minimapPos = 220,
			radius = 80,
		}
	end

	-- Local copy of our DB.
	db = BrokerBFDB
end

-- Enable function.
function BBF:OnEnable()
	if LDB then
		self.obj = LDB:NewDataObject(self.name, {
			type  = "launcher",
			label = "ButtonFacade",
			icon  = "Interface\\Addons\\Broker_ButtonFacade\\Icon",
			tocname = "ButtonFacade",
			OnClick = function(self, button)
				if button == "LeftButton" or button == "RightButton" then
					BF:OpenOptions()
				end
			end,
			OnTooltipShow = function(Tip)
				if not Tip or not Tip.AddLine then return end
				local Group = LBF:Group("ButtonFacade")
				local Skin = Group.SkinID or "Blizzard"
				Tip:AddLine("ButtonFacade")
				Tip:AddLine(L["Click to open the options window."], 1, 1, 1)
				Tip:AddDoubleLine(L["Current Skin:"], "|cffff8000"..Skin.."|r")
			end,
		})
		if Icon then
			Icon:Register(self.name, self.obj, db)
			self:RegisterChatCommand("bficon", function() ToggleIcon() end)
		end
	end
end
