local pName = "Broker_GotMail"
local f = CreateFrame("frame")
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local files = {
	iconNoMail = "Interface\\Minimap\\Tracking\\Mailbox",
	iconNewMail = "Interface\\Minimap\\Tracking\\Mailbox",
}
local colors = {
	["mail_new"] = "|cff00ff00",
	["mail_old"] = "|ccccccccc",
}
local dataobj = ldb:GetDataObjectByName(pName) or ldb:NewDataObject(pName, {
	type = "data source",
	text = "GotMail?",
	icon = files["iconNoMail"],
})
local gotNewMail = false
local ignoreRecentAuctionNotice = false
local ignoreZoningNotice = false
local accurateMailNumbers = false
local gotLoginMail = false
local mailNew = 0
local mailOld = 0
local mailRead = 0
local toolTip = CreateFrame("GameTooltip", "GotMail_Tip")

f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("UPDATE_PENDING_MAIL")
f:RegisterEvent("MAIL_INBOX_UPDATE")
f:RegisterEvent("MAIL_SHOW")
f:RegisterEvent("CHAT_MSG_SYSTEM")

local function updateTooltip()
	if gotNewMail then
		dataobj.text = colors["mail_new"]..L_GOTMAIL["MAIL_NEW"]..FONT_COLOR_CODE_CLOSE
		dataobj.icon = files["iconNewMail"]
	elseif accurateMailNumbers and mailOld > 0 then
		dataobj.text = colors["mail_old"]..L_GOTMAIL["MAIL_OLD"]..FONT_COLOR_CODE_CLOSE
		dataobj.icon = files["iconNoMail"]
	else
		dataobj.text = L_GOTMAIL["MAIL_NONE"]
		dataobj.icon = files["iconNoMail"]
	end
end

local function matchSystemMessage(arg1, arg2)
	arg2 = strsplit("%%s", arg2)
	return strfind(arg1, arg2)
end

local function gotmailInitialize()
	if HasNewMail() then
		gotNewMail = true
		gotLoginMail = true
	end
	if _G["GotMailCounter"] ~= nil then
		mailOld = _G["GotMailCounter"]["mailOld"]
		mailRead = _G["GotMailCounter"]["mailRead"]
	end
	MiniMapMailFrame.Show = MiniMapMailFrame.Hide
	MiniMapMailFrame:Hide()
	updateTooltip()
end

local function gotmailChatmessage(event, arg1)
	if matchSystemMessage(arg1, ERR_AUCTION_OUTBID_S) then
		mailNew = mailNew + 1
		gotNewMail = true
	elseif arg1 == ERR_AUCTION_REMOVED or matchSystemMessage(arg1, ERR_AUCTION_WON_S) then
		ignoreRecentAuctionNotice = true
	end
end

local function gotmailEvent(event, arg1)
	if event == "PLAYER_ENTERING_WORLD" and arg1 == UnitName("player") then
		ignoreZoningNotice = true
		return
	elseif event == "MAIL_INBOX_UPDATE" or event == "MAIL_SHOW" then
		gotNewMail = false
		accurateMailNumbers = true
		mailNew = 0
		mailOld = 0
		mailRead = 0
		for i = 1, (GetInboxNumItems()) do
			local _, _, _, _, _, _, _, _, wasRead, _, _, _, _ = GetInboxHeaderInfo(i)
			if wasRead then
				mailRead = mailRead + 1
			else
				mailOld = mailOld + 1
			end
		end
		_G["GotMailCounter"] = {
			["mailOld"] = mailOld,
			["mailRead"] = mailRead,
		}
	elseif event == "UPDATE_PENDING_MAIL" then
		if ignoreRecentAuctionNotice then
			ignoreRecentAuctionNotice = false
		elseif ignoreZoningNotice then
			ignoreZoningNotice = false
		elseif HasNewMail() then
			mailNew = mailNew + 1
			gotNewMail = true
		end
	end
end

function dataobj.OnTooltipShow(tip)
	if not tip or not tip.AddLine or not tip.AddDoubleLine then
		return
	end

	local new = mailNew
	local old = mailOld
	if accurateMailNumbers == false then
		if gotLoginMail and old == 0 then
			new = "1(+)"
		elseif gotLoginMail then
			new = "?"
		end
	end

	tip:AddDoubleLine(colors["mail_new"]..L_GOTMAIL["LIST_NEW"]..FONT_COLOR_CODE_CLOSE, new)
	tip:AddDoubleLine(colors["mail_old"]..L_GOTMAIL["LIST_OLD"]..FONT_COLOR_CODE_CLOSE, old)
	tip:AddDoubleLine(colors["mail_old"]..L_GOTMAIL["LIST_READ"]..FONT_COLOR_CODE_CLOSE, mailRead)
end

f:SetScript("OnEvent", function(self, event, arg1, ...)
	if event == "ADDON_LOADED" and arg1 == pName then
		gotmailInitialize()
	elseif event == "CHAT_MSG_SYSTEM" then
		gotmailChatmessage(event, arg1)
	else
		gotmailEvent(event, arg1)
	end
	updateTooltip()
end)
