--[[-------------------------------------------------------------------------
	DinnerBell
	Copyright (C) 2010  Morsker
	Please contact Morsker through PM on forums.wowace.com.

	Plays a sound in response to Fish Feasts and other feasts.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.
---------------------------------------------------------------------------]]
local addonName = ...
local Addon = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceEvent-3.0")
_G[addonName] = Addon

-- GLOBALS: InCombatLockdown
-- GLOBALS: PlaySoundFile
-- GLOBALS: AnimTimerFrame
-- GLOBALS: GetCVar
-- GLOBALS: SetCVar
local _G = _G

Addon.soundPath = "Sound\\Creature\\DeathbringerSaurfang\\IC_Dsaurfang_Summon01.wav"

Addon.feastIDs = {
	[57301] = 1, -- Great Feast
	[57426] = 1, -- Fish Feast
	[66476] = 1, -- Bountiful Feast
	[87643] = 1, -- Broiled Dragon Feast
	[87644] = 1, -- Seafood Magnifique Feast
	[87915] = 1, -- Goblin Barbecue
}
local feastIDs = Addon.feastIDs

function Addon:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventtype, ...)
	if eventtype == "SPELL_CAST_START" then
		local _,_,_,_,_,_,spellid = ...
		if feastIDs[spellid] then
			if _G.FixModSounds then
				PlaySoundFile(self.soundPath)
			else
				self.soundPlayer:Play()
			end
		end
	end
end

function Addon:PLAYER_REGEN_DISABLED()
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Addon:PLAYER_REGEN_ENABLED()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Addon:OnInitialize()
	self.OnInitialize = nil

	-- Create self.soundPlayer "Animation" that plays the sound and restores sound settings 1 second later.
	do
		local real_Sound_EnableSFX
		local real_Sound_SFXVolume
		
		local function StartSound()
			real_Sound_EnableSFX = GetCVar("Sound_EnableSFX")
			real_Sound_SFXVolume = GetCVar("Sound_SFXVolume")
			SetCVar("Sound_EnableSFX", 1)
			SetCVar("Sound_SFXVolume", 1.0)
			PlaySoundFile(self.soundPath)
		end
		
		local function DoneSound()
			SetCVar("Sound_EnableSFX", real_Sound_EnableSFX)
			SetCVar("Sound_SFXVolume", real_Sound_SFXVolume)
		end

		self.soundPlayer = AnimTimerFrame:CreateAnimationGroup()
		self.soundPlayer:SetScript("OnPlay", StartSound)
		local anim = self.soundPlayer:CreateAnimation("Animation")
		anim:SetDuration(1.0)
		anim:SetOrder(1)
		anim:SetScript("OnFinished", DoneSound)
	end
end

function Addon:OnEnable()
	if not InCombatLockdown() then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function Addon:OnDisable()
end
