local MAJOR, MINOR = "LibAnchor", "2";
local LibAnchor = LibStub:NewLibrary(MAJOR, MINOR);

if not LibAnchor then
	return;
end

local Dewdrop = LibStub("Dewdrop-2.0");

--------------------------------------------------------------------------------
local Anchor_Backdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 8, edgeSize = 12,
	insets = {
		left = 2,
		right = 2,
		top = 2,
		bottom = 2,
	},
};

local Anchor = {	--template
	anchorpoint = "CENTER",
	refframe = UIParent,
	refpoint = "CENTER",
	anchorposx = 0,
  anchorposy = 0,
	sizeW = 40,
	sizeH = 40,
	strata = "BACKGROUND",
	alpha = 1,
	scale = 1,

	movable = 1,
	resizeable = 0,
	clampedToScreen = 1,
};

--------------------------------------------------------------------------------LibAnchor
function LibAnchor:NewAnchor(parent, anchor)
	return Anchor:_newAnchor(parent, anchor);
end

local dummy = function() end;

function LibAnchor:AddAlignmentMenu(anchor)
	if not anchor or not anchor:GetFrame() then return end
	local f = anchor:GetFrame();
	anchor:AddMenuEntry({
		text = "Center",
		notCheckable = true,
		func = function()
			f:ClearAllPoints();
			f:SetPoint("CENTER");
			anchor:_updateLoc();
		end,
		tooltipFunc = dummy,
		closeWhenClicked = true,
	});
	anchor:AddMenuEntry({
		text = "Center Horizontally",
		notCheckable = true,
		func = function()
			local y = f:GetTop();
			f:ClearAllPoints();
			f:SetPoint("BOTTOM", 0, y);
			anchor:_updateLoc();
		end,
		tooltipFunc = dummy,
		closeWhenClicked = true,
	});
	anchor:AddMenuEntry({
		text = "Center Vertically",
		notCheckable = true,
		func = function()
			local x = f:GetLeft();
			f:ClearAllPoints();
			f:SetPoint("LEFT", x, 0);
			anchor:_updateLoc();
		end,
		tooltipFunc = dummy,
		closeWhenClicked = true,
	});
end

function LibAnchor:AddSepartor(anchor)
	if not anchor or not anchor:GetFrame() then return end
	local f = anchor:GetFrame();
	anchor:AddMenuEntry({
		text = "",
		isTitle = true,
		textHeight = 5,
		tooltipFunc = dummy,
		closeWhenClicked = true,
	});
end
--------------------------------------------------------------------------------Anchor
--private
function Anchor:_newAnchor(parent, anchor)
	anchor = anchor or {};
	setmetatable(anchor, self);
	self.__index = self;

	anchor:_createVisual(parent);
	
	return anchor;
end

function Anchor:_updateLoc()
	local f = self:GetFrame();
	self.anchorposx = f:GetLeft() + f:GetWidth() / 2;
	self.anchorposy = f:GetTop() - f:GetHeight() / 2;
	self.anchorpoint = "CENTER";
	self.refpoint = "BOTTOMLEFT";
end

local function _dewdropMenu(anchor)
	if not Dewdrop:IsOpen() or not anchor or not anchor:GetFrame() then return	end
	Dewdrop:FeedTable(anchor:GetFrame().menuInfo);
end

function Anchor:_createVisual(parent)
	local f = CreateFrame("Frame", nil, parent);
	f.menuInfo = {};
	f:SetMovable(true);
	f:SetResizable(true);
	f:UnregisterAllEvents();

  local adjuster = CreateFrame("Button", nil, f);
  adjuster:SetBackdrop(Anchor_Backdrop);
	adjuster:SetBackdropColor(0.1,1.0,0.2,1);
	adjuster:SetFrameStrata("DIALOG");
  adjuster:SetAllPoints(f);
  f.adjuster = adjuster;

  --scripts
	adjuster:RegisterForDrag("LeftButton");
	adjuster:SetScript("OnDragStart", function()
		f:ClearAllPoints();
		f:StartMoving();
	end);
	adjuster:SetScript("OnDragStop", function()
		f:StopMovingOrSizing();
		self:_updateLoc();
	end);
	--adjuster:RegisterForClicks("RightButtonUp");
	--adjuster:SetScript("OnClick", function()
		--if adjuster:IsShown() and #f.menuInfo > 0 then
			--local cursorX, cursorY = GetCursorPosition();
			--Dewdrop:Open(adjuster, "cursorX", cursorX, "cursorY", cursorY, "children", function()
				--_dewdropMenu(self);
			--end);
		--end
	--end);

	self._getFrame = function()
		return f; 
	end;

	self:Refresh();

	return f;
end

function Anchor:_getFrame()
	return nil;
end

--public
function Anchor:GetFrame()
	return self:_getFrame() or self:_createVisual();
end

function Anchor:SetMovable(movable)
	self.movable = movable and 1 or 0;
	self:Refresh();
end

function Anchor:AddMenuEntry(info)
	tinsert(self:GetFrame().menuInfo, info);
	Dewdrop:Refresh();
end

function Anchor:AddMenuEntries(infos)
	for _, info in pairs(infos) do
		tinsert(self:GetFrame().menuInfo, info);
	end
	Dewdrop:Refresh();
end

function Anchor:Release()
	self:GetFrame():Hide();
end

function Anchor:Refresh()
	f = self:GetFrame();
	f:SetPoint(self.anchorpoint, self.refframe, self.refpoint, self.anchorposx, self.anchorposy);
	f:SetWidth(self.sizeW);
  f:SetHeight(self.sizeH);
  f:SetFrameStrata(self.strata);
  f:SetAlpha(self.alpha);
  f:SetScale(self.scale);
	f:SetClampedToScreen(self.clampedToScreen > 0);
	if self.movable > 0 then
		f.adjuster:Show();
	else
		f.adjuster:Hide();
	end
end

--------------------------------------------------------------------------------