local MAJOR, MINOR = "LibMessGUI", "3";
local MessGUI = LibStub:NewLibrary(MAJOR, MINOR);

if not MessGUI then
	return;
end

-----------------------------------------------------------------------------------------constants
local Btn_Backdrop = {
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	edgeSize = 12,
};

local Frame_Backdrop = {
  bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
  tile = true,
  tileSize = 32,
  edgeSize = 32,
  insets = { top = 8, bottom = 8, left = 8, right = 8, },
};

local Editor_Backdrop = {
	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
	tile = true,
  tileSize = 32,
  insets = { top = 4, bottom = 4, left = 4, right = 4, },
  edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
  edgeSize = 16,
};

local MACRO_MAXLETTERS = 1024;

-----------------------------------------------------------------------------------------ColorPickerFrame
local function ShowColorPicker(r, g, b, a, changedCallback)
	HideUIPanel(ColorPickerFrame);
	ColorPickerFrame:SetFrameStrata("FULLSCREEN_DIALOG");
	ColorPickerFrame.hasOpacity = a;
	ColorPickerFrame.opacity = 1 - (a or 0);
	ColorPickerFrame.previousValues = {r, g, b, a};
	ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = changedCallback, changedCallback, changedCallback;
	ColorPickerFrame:SetColorRGB(r, g, b);
	ShowUIPanel(ColorPickerFrame);
end

-----------------------------------------------------------------------------------------Buttons
local function CreateLabelButton(parent, text, font, size, outLined)
	local btn = CreateFrame("Button", nil, parent);
	btn:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight");
  local label = btn:CreateFontString(nil, "ARTWORK", (not font) and "GameFontNormalSmall" or nil);
  label:SetPoint("CENTER");
  if font then
  	size = size or 12;
		outLined = outLined or "OUTLINE";
  	label:SetFont(font, size, outLined);
  end
  --inlines
  btn.SetText = function(self, text)
  	label:SetText(text);
  	btn:SetWidth(label:GetStringWidth());
  	btn:SetHeight(label:GetStringHeight());
  end

	btn.GetText = function(self)
		return label:GetText();
	end

	--summary
  btn:SetText(text or "");

  return btn;
end

local function CreateIconButton(parent, textureName)
	local btn = CreateFrame("Button", nil, parent);
	btn:SetBackdrop(Btn_Backdrop);
	btn:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight");

	local t = btn:CreateTexture(nil, "BACKGROUND");
	t:SetTexture(textureName);
	t:SetPoint("TOPLEFT", 2, -2);
	t:SetPoint("BOTTOMRIGHT", -2, 2);
	btn.texture = t;

	hooksecurefunc(btn, "Enable", function(self)
		btn.texture:SetDesaturated(false);
	end);
	hooksecurefunc(btn, "Disable", function(self)
		btn.texture:SetDesaturated(true);
	end);

	return btn;
end

local function CreateMixButton(parent, texture, textureCoord, text, fontObject, iconJustifyH, textJustifyH)
	local _GAP = 5;
	iconJustifyH = iconJustifyH or "LEFT";
	textJustifyH = textJustifyH or "RIGHT";
	
	local btn = CreateFrame("Button", nil, parent);
	btn:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight");
	local icon = btn:CreateTexture(nil, "BACKGROUND");
	icon:SetPoint(iconJustifyH);
  local label = btn:CreateFontString(nil, "ARTWORK");
  label:SetPoint(textJustifyH);

  --inlines
  btn.SetIcon = function(self, texture, textureCoord)
  	textureCoord = textureCoord or {0, 1, 0, 1,};
  	icon:SetTexture(texture);
  	if texture then
  		icon:SetTexCoord(unpack(textureCoord));
  	end
  	self:adjust();
  end
	btn.GetIcon = function(self)
		return icon:GetTexture();
	end
	btn.SetText = function(self, text, fontObject)
		fontObject = fontObject or "GameFontNormal";
		label:SetFontObject(fontObject);
  	label:SetText(text);
  	self:adjust();
  end
	btn.GetText = function(self)
		return label:GetText();
	end
	btn.adjust = function(self)
		local h = max(16, label:GetStringHeight());
		icon:SetWidth(h);
		icon:SetHeight(h);
		btn:SetWidth(label:GetStringWidth() + h + _GAP);
  	btn:SetHeight(h);
	end
	--scripts
	hooksecurefunc(btn, "Enable", function(self)
		icon:SetDesaturated(false);
	end);
	hooksecurefunc(btn, "Disable", function(self)
		icon:SetDesaturated(true);
	end);
	--summary
	btn:SetIcon(texture, textureCoord);
  btn:SetText(text, fontObject);

  return btn;
end

local function CreateClickEditor(parent, inset)
	inset = inset or 0;
	
	local f = CreateFrame("Frame", nil, parent);
	local btn = CreateFrame("Button", nil, f);
	btn:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight");
	btn:SetPoint("TOPLEFT", inset, -inset);
	btn:SetPoint("BOTTOMRIGHT", -inset, inset);
  local label = btn:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
  label:SetAllPoints(btn);
  label:SetJustifyH("LEFT");
	local editbox = CreateFrame("EditBox", nil, f);
	editbox:SetAllPoints(btn);
	editbox:SetAutoFocus(true);
	editbox:SetMultiLine(false);
	editbox:SetFontObject("ChatFontNormal");
	editbox:Hide();
	--setscripts
	btn:SetScript("OnClick", function()
		editbox:SetText(label:GetText());
		btn:Hide();
		editbox:Show();
		editbox:SetFocus();
		f:SetBackdrop(Editor_Backdrop);
	end);
	editbox:SetScript("OnEnterPressed", function()
		label:SetText(editbox:GetText());
		btn:Show();
		editbox:Hide();
		f:SetBackdrop(nil);
	end);
	editbox:SetScript("OnEscapePressed", function()
		btn:Show();
		editbox:Hide();
		f:SetBackdrop(nil);
	end);
	editbox:SetScript("OnEditFocusLost", function()
		btn:Show();
		editbox:Hide();
		f:SetBackdrop(nil);
	end);
	--inlines
	f.SetText = function(self, text)
		label:SetText(text);
	end
	
	f.GetText = function(self)
		return label:GetText();
	end
	
	f.GetPorperSize = function(self, text)
		local w, h;
		if text then
			local tmp = label:GetText();
			label:SetText(text);
			w = label:GetStringWidth() + inset;
			h = label:GetStringHeight() + inset;
			label:SetText(tmp);
		else
			w = label:GetStringWidth() + inset;
			h = label:GetStringHeight() + inset;
		end
		
		return w, h;
	end
	--
	return f;
end

local function CreateTextField(parent, callback, inset, fontObject)
	inset = inset or {4, 4, 0, 0};
	fontObject = fontObject or "ChatFontNormal";

	local tf = CreateFrame("EditBox", nil, parent);
	tf:SetFontObject(fontObject);
	tf:SetText("");
	tf:SetAutoFocus(false);
	tf:SetMultiLine(false);
	tf:SetBackdrop(Editor_Backdrop);
	tf:SetHeight(select(2, tf:GetFont()) + 8 + inset[3] + inset[4]);
	tf:SetTextInsets(unpack(inset));
	--setscripts
	tf:SetScript("OnEditFocusGained", function(self)
		self.backup = self:GetText();
	end);
	tf:SetScript("OnEscapePressed", function(self)
		self:ClearFocus();
	end);
	tf:SetScript("OnEnterPressed", function(self)
		self.confirm = 1;
		self:ClearFocus();
	end);
	tf:SetScript("OnEditFocusLost", function(self)
		if self.confirm then
			self.confirm = nil;
			if callback then
				callback(self, self:GetText());
			end
		else
			self:SetText(self.backup);
		end
	end);
	--
	return tf;
end

local function CreateColorChooserBtn(parent, callback)
	local btn =  CreateFrame("Button", nil, parent);
 	btn:SetBackdrop(Btn_Backdrop);
 	btn:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight");
 	btn:SetWidth(32);
 	btn:SetHeight(32);

 	btn.t = btn:CreateTexture(nil, "BACKGROUND");
 	btn.t:SetPoint("TOPLEFT", 4, -4);
 	btn.t:SetPoint("BOTTOMRIGHT", -4, 4);
 	
 	btn.astr = btn:CreateFontString(nil, "ARTWODRK", "GameFontWhiteTiny");
 	btn.astr:SetPoint("CENTER");
 	
 	btn.callback = callback;
 	--scripts
 	btn:SetScript("OnClick", function(self)
 		ShowColorPicker(self.r, self.g, self.b, self.a, function(cancel)
 			local newR, newG, newB, newA;
 			if cancel then
 				newR, newG, newB, newA = unpack(cancel);
 			else
 				newR, newG, newB = ColorPickerFrame:GetColorRGB();
 				newA = 1 - OpacitySliderFrame:GetValue();
 				if self.callback then
 					self:callback(newR, newG, newB, newA);
 				end
 			end
 			self:SetColor(newR, newG, newB, newA);
 		end);
 	end);
 	--inlines
 	btn.SetColor = function(self, r, g, b, a)
 		btn.t:SetTexture(r, g, b);
 		self.r, self.g, self.b, self.a = r, g, b, a;
 		btn.astr:SetText(format("%d%%", a * 100));
 		btn.astr:SetTextColor(1 - r, 1 - g, 1 - b, 1);
 	end
 	btn.GetColor = function(self)
 		return self.r, self.g, self.b, self.a;
 	end
 	--
 	return btn;
end
-----------------------------------------------------------------------------------------Frames
local function CreateContainerFrame(parent, frameStrata)
	frameStrata = frameStrata or "HIGH";
	--body
	local f = CreateFrame("Frame", nil, parent);
	f:SetFrameStrata(frameStrata);
	f:SetMovable(true);
	f:SetResizable(true);
	f:SetClampedToScreen(true);
	f:SetToplevel(true);
	f:SetBackdrop(Frame_Backdrop);
	f:SetMinResize(100, 250);
	--header
	local header = CreateFrame("Button", "$ParentHeader", f);
	header:SetWidth(100);
	header:SetHeight(50);
	header:SetPoint("BOTTOM", f, "TOP", 0, -20);
	header:RegisterForDrag("LeftButton");
	header:SetScript("OnDragStart", function(self)
		f:StartMoving();
	end);
	header:SetScript("OnDragStop", function(self)
		f:StopMovingOrSizing();
	end);
	local ht = header:CreateTexture(nil, "BACKGROUND");
	ht:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header");
	ht:SetTexCoord(0.21875,0, 0.21875,0.625, 0.78125,0, 0.78125,0.625);
  ht:SetAllPoints(header);
  local headerString = header:CreateFontString(nil, "ARTWODRK", "GameFontNormal");
  headerString:SetPoint("CENTER");
  headerString:SetText("ContainerFrame");
  --close btn
  local cbtn = CreateFrame("Button", "$ParentCloseBtn", f, "UIPanelCloseButton");
	cbtn:SetPoint("TOPRIGHT", -3, -3);
	cbtn:SetHeight(25);
	cbtn:SetWidth(25);
	cbtn:SetScript("OnClick", function(self)
		f:Hide();
	end);
	f.CloseBtn = cbtn;
	--inlines
	f.SetTitle = function(self, text)
		headerString:SetText(text or "ContainerFrame");
		header:SetWidth(headerString:GetStringWidth() + 60);
		header:SetHeight(headerString:GetStringHeight() + 30);
		self:SetMinResize(header:GetWidth(), header:GetHeight() * 5);
	end
	--resize btn
	local sbtn = CreateFrame("Button", "$ParentSizeBtn", f);
	sbtn:SetWidth(22);
	sbtn:SetHeight(21);
	sbtn:SetPoint("BOTTOMRIGHT");
	local t = sbtn:CreateTexture(nil, "BACKGROUND");
	t:SetTexture("Interface\\BUTTONS\\UI-AutoCastableOverlay");
	t:SetTexCoord(0.619, 0.760, 0.612, 0.762);
	t:SetDesaturated(true);
	t:SetPoint("TOPLEFT");
	t:SetPoint("BOTTOMRIGHT", sbtn, "TOPLEFT", 12, -12);
	sbtn:SetScript("OnMouseDown", function(self)
		f:StartSizing("BOTTOMRIGHT");
	end);
	sbtn:SetScript("OnMouseUp", function(self)
		f:StopMovingOrSizing();
	end);
	sbtn:SetScript("OnEnter", function(self)
		t:SetDesaturated(false);
	end);
	sbtn:SetScript("OnLeave", function(self)
		t:SetDesaturated(true);
	end);
	
	return f;
end

local function CreateHDistributeFrame(parent, insets, frameType)
	frameType = frameType or "Frame";
	local f = CreateFrame("Frame", nil, parent);
	f.entryIndex = {};	-- index	:	key
	f.entries = {};			-- key		:	entryFrame
	f.insets = insets or { left = 0, right = 0, top = 0, bottom = 0, };
	--inlines
	f.ClearAll = function(self)
		for _, entry in pairs(self.entries) do
			entry:Hide();
		end
		wipe(self.entryIndex);
		wipe(self.entries);
	end
	f.AddEntry = function(self, key, entry)
		if type(entry) == "string" then
			local text = entry;
			entry = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
			entry:SetText(text);
		else
			entry:SetParent(self);
		end
		tinsert(self.entryIndex, key);
		self.entries[key] = entry;
		self:redistribute();
	end
	f.DelEntry = function(self, key)
		local entry;
		if self.entries[key] then
			entry = self.entries[key];
			self.entries[key]:Hide();
			self.entries[key] = nil;
			for i, k in ipairs(self.entryIndex) do
				if key == k then
					tremove(self.entryIndex, i);
					break;
				end
			end
		end
		self:redistribute();
		return entry;
	end
	f.GetEntry = function(self, key)
		return self.entries[key];
	end
	f.GetEntries = function(self)
		return self.entries;
	end
	f.redistribute = function(self)
		local key;
		local s, e = self.insets.left, self:GetWidth() - self.insets.right;
		local section = #self.entryIndex;
		local maxH = 0;
		if section > 0 then
			key = self.entryIndex[1];
			self.entries[key]:ClearAllPoints();
			self.entries[key]:SetPoint("LEFT", 0, 0);
			maxH = max(maxH, self.entries[key]:GetHeight());
		end
		if section > 1 then
			key = self.entryIndex[section];
			self.entries[key]:ClearAllPoints();
			self.entries[key]:SetPoint("RIGHT", 0, 0);
			maxH = max(maxH, self.entries[key]:GetHeight());
		end
		if section > 2 then
			local gap = (e - s) / (section - 1);
			for i = 2, section - 1 do
				key = self.entryIndex[i];
				self.entries[key]:ClearAllPoints();
				self.entries[key]:SetPoint("LEFT", s + (i - 1) * gap - self.entries[key]:GetWidth() / 2, 0);
				maxH = max(maxH, self.entries[key]:GetHeight());
			end
		end
		self:SetHeight(maxH + self.insets.top + self.insets.bottom);
	end
	--scripts
	f:SetScript("OnSizeChanged", function(self, w, h)
		if w == self.w then	return end
		self.w = w;
		self:redistribute();
	end);
	--
	return f;
end

-----------------------------------------------------------------------------------------Slider
local sliderCnt = 0;

local function CreateSlider(parent, callback)
	sliderCnt = sliderCnt + 1;
	local f = CreateFrame("Frame", "Slider_"..sliderCnt, parent);
	f:SetWidth(200);
	f:SetHeight(32);
	f.formatStr = "%d";
	f.callback = callback;

	f.edit = CreateFrame("EditBox", "$ParentEditbox", f, "InputBoxTemplate");
	f.edit:SetWidth(50);
	f.edit:SetHeight(20);
	f.edit:SetPoint("LEFT");
	f.edit:SetAutoFocus(false);
	f.edit:SetMultiLine(false);
	f.edit:SetFontObject("GameFontHighlight");
	f.edit:SetJustifyH("CENTER");
	f.slider = CreateFrame("Slider", "$ParentSlider", f, "OptionsSliderTemplate");
	f.slider:SetValueStep(1);
	f.slider:SetOrientation("HORIZONTAL");
	f.slider:SetPoint("RIGHT");
	f.slider:SetPoint("BOTTOM");
	f.slider:SetPoint("LEFT", f.edit, "RIGHT", 5, 0);
	local sliderName = f.slider:GetName();
	f.title = _G[sliderName.."Text"];
	_G[sliderName.."Low"]:SetText("");
	_G[sliderName.."High"]:SetText("");

	--scripts
	f.edit:SetScript("OnEnterPressed", function(self)
		self.confirm = true;
		self:ClearFocus();
	end);
	f.edit:SetScript("OnEscapePressed", function(self)
		self:ClearFocus();
	end);
	f.edit:SetScript("OnEditFocusLost", function(self)
		local parent = self:GetParent();
		if self.confirm then
			self:GetParent().slider:SetValue(self:GetNumber());
		else
			self:SetText(format(parent.formatStr, parent.slider:GetValue()));
		end
		self.confirm = nil;
	end);
	f.slider:SetScript("OnValueChanged", function(self, value)
		local parent = self:GetParent();
		parent.edit:SetText(format(parent.formatStr, value));
		if parent.callback then
			parent:callback(value);
		end
	end);
	--inlines
	f.SetTitle = function(self, text)
		self.title:SetText(text);
	end
	f.SetValue = function(self, value)
		self.slider:SetValue(value);
	end
	f.GetValue = function(self)
		return self.slider:GetValue();
	end
	f.SetValueStep = function(self, step)
		self.slider:SetValueStep(step);
		if step % 1 > 0 then
			local deciCnt = 1;
			while step % 1 > 0 do
				step = step * 10;
				deciCnt = deciCnt + 1;
			end
			self.formatStr = "%."..deciCnt.."f";
		else
			self.formatStr = "%d";
		end
	end
	f.SetMinMaxValues = function(self, min, max)
		self.slider:SetMinMaxValues(min, max);
	end
	f.GetMinMaxValues = function(self)
		return self.slider:GetMinMaxValues();
	end
	f.SetCallback = function(self, callback)
		self.callback = callback;
	end
	f.SetEditable = function(self, editable)
		self.edit:EnableMouse(editable);
	end
	f.IsEditable = function(self)
		return not self.blocker:IsShown();
	end
	--
	return f;
end

-----------------------------------------------------------------------------------------ScrollPane
local function CreateScrollPane(parent, stepSize, inset)
	local scrollPane = CreateFrame("Frame", "$ParentScrollFrame", parent);
	scrollPane:EnableMouse(true);
  scrollPane.ENTRY_HEIGHT = stepSize or 16;
  scrollPane.INSET = inset or 0;

  local scrollbar = CreateFrame("Slider", "$ParentScrollBar", scrollPane, "UIPanelScrollBarTemplate");
  scrollbar:SetPoint("TOPRIGHT", -2, -20);
  scrollbar:SetPoint("BOTTOMRIGHT", -2, 20);
  scrollbar:SetValueStep(1);
  scrollbar:SetMinMaxValues(0, 0);
	scrollbar:Hide();
	scrollPane.scrollbar = scrollbar;

  local viewport = CreateFrame("ScrollFrame", "$ParentViewport", scrollPane);
  viewport:SetPoint("TOPLEFT", scrollPane.INSET, -scrollPane.INSET);
  viewport:SetPoint("BOTTOM", 0, scrollPane.INSET);
  viewport:SetPoint("RIGHT", -scrollPane.INSET, 0);
  viewport:EnableMouseWheel(true);
  scrollPane.viewport = viewport;

  local content = CreateFrame("Frame", "$ParentContent", viewport);
  content:SetPoint("TOPLEFT", 0, 0);
  content:SetPoint("TOPRIGHT", 0, 0);
  scrollPane.content = content;
  viewport:SetScrollChild(content);

	--setscripts
	hooksecurefunc(scrollbar, "SetMinMaxValues", function(self, min, max)
		if floor(max) == floor(min) then	--better round it up
			scrollbar:Hide();
			viewport:SetPoint("RIGHT", -scrollPane.INSET, 0);
		else
			scrollbar:Show();
			viewport:SetPoint("RIGHT", scrollbar, "LEFT", -scrollPane.INSET, 0);
		end
	end);

	scrollbar:SetScript("OnValueChanged", function(self, value)
  	content:ClearAllPoints();
  	content:SetPoint("TOPLEFT", 0, value);
  	content:SetPoint("TOPRIGHT", 0, value);
  end);
	viewport:SetScript("OnMouseWheel", function(self, value)
		local smin, smax = scrollbar:GetMinMaxValues();
		value = value < 0 and scrollPane.ENTRY_HEIGHT or -scrollPane.ENTRY_HEIGHT; --reverse!
	  local offset = min(max(scrollbar:GetValue() + value, smin), smax);
	  scrollbar:SetValue(offset);
  end);
	content:SetScript("OnSizeChanged", function(self, w, h)
		local maxValue = max(0, h - viewport:GetHeight());
		maxValue = ceil(maxValue / scrollPane.ENTRY_HEIGHT) * scrollPane.ENTRY_HEIGHT;
  	scrollbar:SetMinMaxValues(0, maxValue);
	end);
	viewport:SetScript("OnSizeChanged", function(self, w, h)
		local maxValue = max(0, content:GetHeight() - h);
		maxValue = ceil(maxValue / scrollPane.ENTRY_HEIGHT) * scrollPane.ENTRY_HEIGHT;
  	scrollbar:SetMinMaxValues(0, maxValue);
	end);
	--inlines
	scrollPane.SetContentHeight = function(self, height)
		content:SetHeight(height);
	end
	
	scrollPane.GetContentHeight = function(self)
		return content:GetHeight();
	end

	scrollPane.SetScrollPosition = function(self, pos)
		local smin, smax = scrollbar:GetMinMaxValues();
		scrollbar:SetValue(min(max(pos, smin), smax));
	end

	scrollPane.SetIncrementalScrollPosition = function(self, pos, offsetH)
		local smin, smax = scrollbar:GetMinMaxValues();
		local viewY, viewH = scrollbar:GetValue(), viewport:GetHeight();
		if pos < viewY then
			scrollPane:SetScrollPosition(pos);
		elseif pos + offsetH > viewY + viewH then
			scrollPane:SetScrollPosition(pos + offsetH - viewH);
		else
			--in view, do nothing
		end
	end

	scrollPane.GetScrollPosition = function(self)
		return scrollbar:GetValue();
	end
	
	scrollPane.GetScrollMinMax = function(self)
		return scrollbar:GetMinMaxValues();
	end
	--summary
	scrollbar:SetValue(0);

  return scrollPane;
end

local function CreateScrollTextArea(parent)
	local scrollPane = CreateScrollPane(parent, 14, 4);
	scrollPane:SetBackdrop(Editor_Backdrop);

	local editBox = CreateFrame("EditBox", nil, scrollPane.content);
	editBox:SetAllPoints(scrollPane.content);
	editBox:SetFontObject(ChatFontNormal);
	editBox:SetMaxLetters(MACRO_MAXLETTERS);
	editBox:SetMultiLine(true);
	editBox:SetAutoFocus(false);
	scrollPane.editBox = editBox;
	--setscripts
	editBox:SetScript("OnTextChanged", function()
		--size
		scrollPane:SetContentHeight(max(editBox:GetHeight(), scrollPane.viewport:GetHeight()));
	end);
	editBox:SetScript("OnCursorChanged", function(self, x, y, width, height)
		--follow editing cursor postion
		scrollPane:SetIncrementalScrollPosition(abs(y), height);
	end);
	--inlines
	scrollPane.SetEditorText = function(self, text)
		if text ~= editBox:GetText() then
			editBox:SetText(text);
			editBox:SetCursorPosition(0);
		end
	end

	scrollPane.GetEditorText = function(self)
		return editBox:GetText();
	end

	--summary
	scrollPane:SetContentHeight(scrollPane:GetHeight());
	return scrollPane;
end

-----------------------------------------------------------------------------------------IconChooser
local IconChooser;
local NumLines = 4;
local NumColumns = 5;
local stringToIndex;

function createIconBtn(parent)
  local btn = CreateFrame("Button", nil, parent);
  btn:SetHeight(36);
  btn:SetWidth(36);
  btn:SetScript("OnEnter", function() btn.HighlightTexture:Show() end);
  btn:SetScript("OnLeave", function() btn.HighlightTexture:Hide() end);

  btn.BackgroundTexture = btn:CreateTexture(nil, "BACKGROUND");
  btn.BackgroundTexture:SetTexture("Interface\\Buttons\\UI-EmptySlot-Disabled");
  btn.BackgroundTexture:SetWidth(64);
  btn.BackgroundTexture:SetHeight(64);
  btn.BackgroundTexture:SetPoint("CENTER", btn, "CENTER", 0, -1);

  btn.NormalTexture = btn:CreateTexture(nil, "ARTWORK");
  btn.NormalTexture:SetWidth(36);
  btn.NormalTexture:SetHeight(36);
  btn.NormalTexture:SetPoint("CENTER", btn, "CENTER", 0, 0);

  btn.HighlightTexture = btn:CreateTexture(nil, "HIGHLIGHT");
  btn.HighlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square");
  btn.HighlightTexture:SetWidth(36);
  btn.HighlightTexture:SetHeight(36);
  btn.HighlightTexture:SetPoint("CENTER", btn, "CENTER", 0, 0);
  btn.HighlightTexture:SetBlendMode("ADD");
  btn.HighlightTexture:Hide();

  btn.SelectedTexture = btn:CreateTexture(nil, "OVERLAY");
  btn.SelectedTexture:SetTexture("Interface\\Buttons\\CheckButtonHilight");
  btn.SelectedTexture:SetWidth(36);
  btn.SelectedTexture:SetHeight(36);
  btn.SelectedTexture:SetPoint("CENTER", btn, "CENTER", 0, 0);
  btn.SelectedTexture:SetBlendMode("ADD");
  btn.SelectedTexture:Hide();

  return btn;
end

function createIconChoiceScrollLine(parent)
  local Frame = CreateFrame("Frame", nil, parent);
  local Spacing = 8;
  Frame:SetHeight(36);
  Frame:SetWidth(36 * NumColumns + Spacing * (NumColumns - 1));

  Frame.Icon1 = createIconBtn(Frame);
  Frame.Icon1:SetPoint("TOPLEFT", Frame, "TOPLEFT", 0, 0);
  Frame.Icon1:SetScript("OnClick", function() parent:SetChoice(Frame.LineNum, 1) end);
  for Index = 2, NumColumns do
    Frame["Icon"..Index] = createIconBtn(Frame);
    Frame["Icon"..Index]:SetPoint("TOPLEFT", Frame["Icon"..(Index - 1)], "TOPRIGHT", Spacing, 0);
    Frame["Icon"..Index]:SetScript("OnClick", function() parent:SetChoice(Frame.LineNum, Index) end);
  end

  Frame.HighlightIcon = function(self, IconNum, Hide)
	  self.SelectedIcon = IconNum;
	  for Index = 1, NumColumns do
	    if ( Index == IconNum ) then
	      self["Icon"..Index].SelectedTexture:Show()
	    else
	      self["Icon"..Index].SelectedTexture:Hide()
	    end
	  end
	end;

  Frame.SelectedIcon = 0;
  return Frame;
end

function CreateIconChooser()
	local frame = CreateFrame("Frame", "IconChooser");
	frame.Choice = 0;
	frame:SetPoint("CENTER", UIParent, "CENTER");
	frame:SetHeight(230);
	frame:SetWidth(270);
	frame:SetToplevel(true);
	frame:SetFrameStrata("TOOLTIP");
	frame:EnableMouse(true);
	frame:SetBackdrop(Frame_Backdrop);

	if not stringToIndex then
		stringToIndex = {};
		for Index = 1, GetNumMacroIcons() do
		  stringToIndex[GetMacroIconInfo(Index)] = Index;
		end
	end

  frame.ScrollFrame = CreateFrame("ScrollFrame", "$ParentIconChoiceScrollFrame", frame, "FauxScrollFrameTemplate");
  frame.ScrollFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 18, -18);
  frame.ScrollFrame:SetHeight(168);
  frame.ScrollFrame:SetWidth(212);
  frame.ScrollFrame:SetScript("OnVerticalScroll", function(self, offset)
  	FauxScrollFrame_OnVerticalScroll(self, offset, 36, frame.OnScroll);
  end);

  frame.ScrollLine1 = createIconChoiceScrollLine(frame);
  frame.ScrollLine1:SetPoint("TOPLEFT", frame.ScrollFrame, "TOPLEFT", 0, 0);
  frame.ScrollLine1.LineNum = 1;
  for Index = 2, NumLines do
    frame["ScrollLine"..Index] = createIconChoiceScrollLine(frame);
    frame["ScrollLine"..Index]:SetPoint("TOPLEFT", frame["ScrollLine"..(Index - 1)], "BOTTOMLEFT", 0, -7);
    frame["ScrollLine"..Index].LineNum = Index;
  end

	frame.CancelButton = CreateFrame("Button", nil, frame, "GameMenuButtonTemplate");
  frame.CancelButton:SetWidth(60);
  frame.CancelButton:SetHeight(21);
  frame.CancelButton:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -20, 20);
  frame.CancelButton:SetText("Cancel");
  frame.CancelButton:SetScript("OnClick", function()
  	frame:Hide();
  end);

  frame.AcceptButton = CreateFrame("Button", nil, frame, "GameMenuButtonTemplate");
  frame.AcceptButton:SetWidth(75);
  frame.AcceptButton:SetHeight(24);
  frame.AcceptButton:SetPoint("RIGHT", frame.CancelButton, "LEFT", -10, 0);
  frame.AcceptButton:SetText("Accept");
  frame.AcceptButton:SetScript("OnClick", function()
    if frame.callBack then
    	frame:callBack(frame.Choice);
    end
    frame:Hide();
  end);

  
	frame:Hide();

	--functions
	frame.SetChoice = function(self, Line, Icon)
		local OldLineNum = math.ceil(frame.Choice / NumColumns) - FauxScrollFrame_GetOffset(frame.ScrollFrame);
	  if ( 1 <= OldLineNum and OldLineNum <= NumLines ) then
	    frame["ScrollLine"..OldLineNum]:HighlightIcon(0);
	  end
	  local LineOffset = Line + FauxScrollFrame_GetOffset(frame.ScrollFrame);
	  frame.Choice = (LineOffset - 1) * NumColumns + Icon;
	  frame["ScrollLine"..Line]:HighlightIcon(Icon);
	end

	frame.OnScroll = function()
	  local LineNum, LineOffset;
	  local NumEntries = math.ceil(GetNumMacroIcons() / NumColumns);
	  local NumScrollLines = NumLines;
	  local LineHeight = 36;
	  FauxScrollFrame_Update(frame.ScrollFrame, NumEntries, NumScrollLines, LineHeight);

	  for LineNum = 1, NumScrollLines do
	    LineOffset = LineNum + FauxScrollFrame_GetOffset(frame.ScrollFrame);
	    if LineOffset <= NumEntries then
	      -- If Start of each line <= Icon Choice <= End of each line   ( LineOffset * 5 - 4 <= Choice <= LineOffset * 5)
	      if ( LineOffset * NumColumns - (NumColumns - 1) <= frame.Choice and frame.Choice <= LineOffset * NumColumns ) then
	        frame["ScrollLine"..LineNum]:HighlightIcon(frame.Choice - (LineOffset - 1) * NumColumns);
	      else
	        frame["ScrollLine"..LineNum]:HighlightIcon(0);
	      end
	      for Index = 1, NumColumns do
	        if ( (LineOffset - 1) * NumColumns + Index <= GetNumMacroIcons() ) then
	          frame["ScrollLine"..LineNum]["Icon"..Index].NormalTexture:SetTexture(GetMacroIconInfo((LineOffset - 1) * NumColumns + Index));
	          frame["ScrollLine"..LineNum]["Icon"..Index]:Show();
	        else
	          frame["ScrollLine"..LineNum]["Icon"..Index]:Hide();
	        end
	      end
	      frame["ScrollLine"..LineNum]:Show();
	    else
	      frame["ScrollLine"..LineNum]:Hide();
	    end
	  end
	end
	--inlines
	frame.SetCallback = function(self, callBack)
		frame.callBack = callBack;
	end

	frame.SetSelected = function(self, textureString)
		textureString = textureString or "";
		frame.Choice = stringToIndex[textureString] or stringToIndex[textureString:gsub("%.blp", "")] or 1;

		local NumEntries = math.ceil(GetNumMacroIcons() / NumColumns);
	  local NumScrollLines = NumLines;
	  local LineHeight = 36;
	  FauxScrollFrame_Update(self.ScrollFrame, NumEntries, NumScrollLines, LineHeight);

	  local Offset = math.ceil(self.Choice / NumColumns) - 1;
	  _G[frame.ScrollFrame:GetName().."ScrollBar"]:SetValue(Offset * LineHeight); -- Triggers OnVerticalScroll only if the value changes, so need to call OnScroll
	end
	--event
	frame:SetScript("OnShow", function()
	  frame:OnScroll();
	end);

  return frame;
end

-----------------------------------------------------------------------------------------MacroEditor
local MacroEditor;

local function CreateMacroEditor()
	local f = CreateFrame("Frame", "MacroEditor");
	f:SetFrameStrata("TOOLTIP");
	f:SetBackdrop(Frame_Backdrop);
	f:SetToplevel(true);

	local iconBtn = CreateFrame("Button", "$ParentIconBtn", f, "ActionButtonTemplate");
	iconBtn:SetPoint("TOPLEFT", 20, -20);
	iconBtn:SetWidth(32);
	iconBtn:SetHeight(32);
	local iconTexture = _G[iconBtn:GetName().."Icon"];
	iconBtn:SetScript("OnClick", function()
		local iconChooser = MessGUI:OpenIconChooser(iconTexture:GetTexture(), function(self, id)
			iconTexture:SetTexture(GetMacroIconInfo(id));
		end);
		iconChooser:SetPoint("TOPLEFT", iconBtn, "TOPRIGHT", 0, 0);
	end);

	local nameEditor = CreateTextField(f);--CreateClickEditor(f, 2);
	nameEditor:SetPoint("TOPLEFT", iconBtn, "TOPRIGHT", 10, 0);
	nameEditor:SetPoint("RIGHT", f, "RIGHT", -20, 0);
	nameEditor:SetHeight(24);

	local scrollEditBox = CreateScrollTextArea(f);
	scrollEditBox:SetPoint("TOPLEFT", nameEditor, "BOTTOMLEFT", 0, -10);

	local cancelBtn = CreateFrame("Button", "$ParentCancelBtn", f, "GameMenuButtonTemplate");
  cancelBtn:SetWidth(75);
  cancelBtn:SetHeight(24);
  cancelBtn:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -20, 20);
  cancelBtn:SetText("Cancel");
  cancelBtn:SetScript("OnClick", function()
  	f:Hide();
  end);

	local acceptBtn = CreateFrame("Button", "$ParentAcceptBtn", f, "GameMenuButtonTemplate");
  acceptBtn:SetWidth(75);
  acceptBtn:SetHeight(24);
  acceptBtn:SetPoint("RIGHT", cancelBtn, "LEFT", -10, 0);
  acceptBtn:SetText("Accept");
  acceptBtn:SetScript("OnClick", function()
  	if f.callback then
  		f:callback(nameEditor:GetText(), iconTexture:GetTexture(), scrollEditBox:GetEditorText());
  	end
    f:Hide();
  end);

	local counterLabel = f:CreateFontString(nil, "BACKGROUND", "GameFontWhiteTiny");
	counterLabel:SetPoint("BOTTOMRIGHT", cancelBtn, "TOPRIGHT", 0, 10);
	counterLabel:SetJustifyH("RIGHT");

	local originOnTextChange = scrollEditBox.editBox:GetScript("OnTextChanged");
	scrollEditBox.editBox:SetScript("OnTextChanged", function(...)
		if originOnTextChange then
			originOnTextChange(...);
		end
		counterLabel:SetFormattedText("%5d/%5d", scrollEditBox.editBox:GetNumLetters(), MACRO_MAXLETTERS);
	end);

  scrollEditBox:SetPoint("BOTTOMRIGHT", counterLabel, "TOPRIGHT", 0, 10);
	--inlines
	f.OpenMacroEditor = function(self, name, texture, body, callback)
		nameEditor:SetText(name or "unNamed");
		iconTexture:SetTexture(texture);
		scrollEditBox:SetEditorText(body);
		f.callback = callback;
	end

	--summary
	f:SetWidth(400);
	f:SetHeight(250);
	f:Hide();

	return f;
end

-----------------------------------------------------------------------------------------public func
function MessGUI:GetLabelButton(parent, text, font, size, outLined)
	return CreateLabelButton(parent, text, font, size, outLined);
end

function MessGUI:GetIconButton(parent, textureName)
	return CreateIconButton(parent, textureName);
end

function MessGUI:GetMixButton(parent, texture, textureCoord, text, fontObject, iconJustifyH, textJustifyH)
	return CreateMixButton(parent, texture, textureCoord, text, fontObject, iconJustifyH, textJustifyH);
end

function MessGUI:GetTextField(parent, callback, fontObject)
	return CreateTextField(parent, callback, fontObject);
end

function MessGUI:GetColorChooserBtn(parent, callback)
	return CreateColorChooserBtn(parent, callback);
end

function MessGUI:GetContainerFrame(parent, frameStrata)
	return CreateContainerFrame(parent, frameStrata);
end

function MessGUI:GetHorizontalDistributeFrame(parent, insets, frameType)
	return CreateHDistributeFrame(parent, insets, frameType);
end

function MessGUI:GetSlider(parent, callback)
	return CreateSlider(parent, callback);
end

function MessGUI:GetScrollPane(parent, stepSize, inset)
	return CreateScrollPane(parent, stepSize, inset);
end

function MessGUI:GetScrollTextArea(parent)
	return CreateScrollTextArea(parent);
end

function MessGUI:GetIconChooser()
	if not IconChooser then
		IconChooser = CreateIconChooser();
	end
	return IconChooser;
end

function MessGUI:OpenIconChooser(texture, callbackFunc)
	local chooser = self:GetIconChooser();
	chooser:SetSelected(texture);
	chooser:SetCallback(callbackFunc);
	chooser:Show();
	return chooser;
end

function MessGUI:GetMacroEditor()
	if not MacroEditor then
		MacroEditor = CreateMacroEditor();
	end
	return MacroEditor;
end

function MessGUI:OpenMacroEditor(name, texture, body, callbackFunc)
	local macroEditor = self:GetMacroEditor();
	macroEditor:OpenMacroEditor(name, texture, body, callbackFunc);
	macroEditor:Show();
	return macroEditor;
end

-----------------------------------------------------------------------------------------
function MessGUI:ColorCodeString(text, color)
	if not color then 
		return "FFFFFFFF";
	end
	local prefix = "|c"..string.format("%.2X%.2X%.2X%.2X", (color.a or color.opacity or 1) * 255, (color.r or 0) * 255, (color.g or 0) * 255, (color.b or 0) * 255);
	return prefix..text..FONT_COLOR_CODE_CLOSE;
end