local function am(msg)
	ChatFrame1:AddMessage(msg);
end

-------------------------------------------------------------------------------------------------------------------------
local LibAnchor = LibStub("LibAnchor", true);
local LibRingBar = LibStub("LibRingBar", true);

local _name, _addon = ...;

local DEFAULT_WATCHSPELLID = { 
	47541,	--Death Coil
	1082,		--claw
	772,		--rend
	172,		--currption
	52127, 	--Water Shield
	1752,		--Sinister Strike
	21562, 	--Power Word: Fortitude
	20217, 	--Blessing of Kings
	1449, 	--Arcane Explosion
	136,		--Mend Pet
};

local defaultDB = {
	watchSpells = {},
	anchor = {},
	ring = {
		textureID = "ring_default";
	},
	alpha = 0.7,
	scale = 1,
	beginColor = {r = 1, g = 1, b = 1, a = 1},
	endColor = {r = 1, g = 1, b = 1, a = 1},
	traditionalTrigger = nil,
	smoother = nil,
};

local _DEBUG =	{
	--enabled = true,
	spell = nil,
	max = 0,
	min = 999,
	debugString = nil,
};

-------------------------------------------------------------------------------------------------------------------------
local ANCHOR;
local RING;

local addon = CreateFrame("Frame", nil, UIParent);
addon:UnregisterAllEvents();

local function _stopTimer()
	addon:SetScript("OnUpdate", nil);
	RING:SetMin();
end

local function _counter(self, elapsed)
	local min, max = RING:GetMinMaxValues();
	local newValue = RING:GetValue() + elapsed;
	if max <= newValue then
		_stopTimer();
	else
		RING:SetValue(newValue);
		local b = GCDRingDB.beginColor;
		local e = GCDRingDB.endColor;
		local perc = newValue / (max - min);
		RING:SetRingColor( b.r + perc * (e.r - b.r), b.g + perc * (e.g - b.g),  b.b + perc * (e.b - b.b), b.a + perc * (e.a - b.a));
	end
end

local function _startTimer(start, dur)
	if _DEBUG.enabled then
		_DEBUG.max = math.max(_DEBUG.max, dur);
		_DEBUG.min = math.min(_DEBUG.min, dur);
		_DEBUG.debugString:SetText(format("%s <%.2f>[$.2f]<%.2f>", _DEBUG.spell, _DEBUG.min, dur, _DEBUG.max));
	end

	local min, max = RING:GetMinMaxValues();
	local value = RING:GetValue();

	dur = start + dur - GetTime();
	if dur < 0 then
		return;
	elseif dur == 0 then
		RING:SetValue(0);
	elseif GCDRingDB.smoother and value > 0 then
		value = dur * value / (max - min - value);
		RING:SetMinMaxValues(min, value + dur);
		RING:SetValue(value);
	else
		RING:SetMinMaxValues(0, dur);
		RING:SetValue(0);
	end
	addon:SetScript("OnUpdate", _counter);
end

function addon:CheckWatchSpell()
	_stopTimer();
	self.watchSpell = nil;

	for _, spellName in ipairs(GCDRingDB.watchSpells) do
		if GetSpellInfo(spellName) then
			self.watchSpell = spellName;
			break;
		end
	end
	--default
	local spellName;
	if not self.watchSpell then
		for _, spellID in ipairs(DEFAULT_WATCHSPELLID) do
			spellName = GetSpellInfo(spellID);
			if spellName and GetSpellInfo(spellName) then
				self.watchSpell = spellName;
				break;
			end			
		end
	end
	
	if _DEBUG.enabled then
		_DEBUG.spell = self.watchSpell;
		_DEBUG.max = 0;
		_DEBUG.min = 999;
		_DEBUG.debugString:SetText(self.watchSpell);
	end
end

function addon:ADDON_LOADED(name)
	if name ~= _name then
		return;
	end
	if not GCDRingDB then
		GCDRingDB = CopyTable(defaultDB);
	end
	self:UnregisterEvent("ADDON_LOADED");
	self:RegisterEvent("PLAYER_LOGIN");
end

function addon:PLAYER_LOGIN()
	self:UnregisterEvent("PLAYER_LOGIN");

	GCDRingDB.anchor = GCDRingDB.anchor or {}
	ANCHOR = LibAnchor:NewAnchor(UIParent, GCDRingDB.anchor);
	--anchor menu
	--LibAnchor:AddAlignmentMenu(ANCHOR);
	--LibAnchor:AddSepartor(ANCHOR);
	--ANCHOR:AddMenuEntry({
		--text = "Lock Anchor",
		--notCheckable = true,
		--func = function()
			--_addon:setMovable(false);
		--end,
		--tooltipFunc = dummy,
		--closeWhenClicked = true,
	--});
	--ANCHOR:AddMenuEntry({
		--text = "Open Config",
		--notCheckable = true,
		--func = function()
			--_addon:OpenConfig();
		--end,
		--tooltipFunc = dummy,
		--closeWhenClicked = true,
	--});

	GCDRingDB.ring = GCDRingDB.ring or {
		textureID = "ring_broad/blend",
		sparkID = "empty",
	};
	RING = LibRingBar:NewRingBar(ANCHOR:GetFrame(), GCDRingDB.ring);

	if _DEBUG.enabled then
		_DEBUG.debugString = RING:GetFrame():CreateFontString(nil, "OVERLAY", "GameFontNormal");
		_DEBUG.debugString:SetPoint("CENTER");
	end

	RING:GetFrame():SetAlpha(GCDRingDB.alpha);
	RING:GetFrame():SetScale(GCDRingDB.scale);
	--check watch spell
	self:CheckWatchSpell();
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	self:RegisterEvent("UNIT_SPELLCAST_START");
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("PLAYER_TALENT_UPDATE");
end

function addon:PLAYER_TALENT_UPDATE()
	self:CheckWatchSpell();
end

function addon:ACTIONBAR_UPDATE_COOLDOWN()
	if not self.watchSpell or not GCDRingDB.traditionalTrigger or self.testMode then
		return;
	end
	local start, dur = GetSpellCooldown(self.watchSpell);
	if not dur then
		--do nothing
	elseif dur <= 0 then
		_stopTimer();
	else
		_startTimer(start, dur);
	end
end

function addon:UNIT_SPELLCAST_START(unit, spell)
	self:UNIT_SPELLCAST_SUCCEEDED(unit, spell);
end

function addon:UNIT_SPELLCAST_SUCCEEDED(unit, spell)
	if unit ~= "player" or GCDRingDB.traditionalTrigger or self.testMode then
		return;
	end
	local start, dur = GetSpellCooldown(self.watchSpell or spell);
	if not dur then
		--do nothing
	elseif dur <= 0 then
		_stopTimer();
	elseif not self.watchSpell and dur > 1.5 then
		--do nothing
	else
		_startTimer(start, dur);
	end
end

addon:SetScript("OnEvent", function(self, event, ...)
	local f = self[event];
	if f then
		f(self,...);
	else
		am(tostring(event));
	end
end);

-------------------------------------------------------------------------------------------------------------------------
function _addon:getMovable()
	return ANCHOR.movable > 0;
end

function _addon:setMovable(movable)
	ANCHOR:SetMovable(movable);
end

function _addon:getSmooth()
	return GCDRingDB.smoother;
end

function _addon:setSmooth(smooth)
	GCDRingDB.smoother = smooth;
end

function _addon:getTTrigger()
	return GCDRingDB.traditionalTrigger;
end

function _addon:setTTrigger(trigger)
	GCDRingDB.traditionalTrigger = trigger;
end

function _addon:SetTestMode(on)
	if on then
		RING:SetMax();
	else
		addon:CheckWatchSpell();
	end
	self.testMode = on;
end

function _addon:getRingCfg(var)
	return RING[var];
end

function _addon:setRingCfg(var, value)
	RING[var] = value;
	if var == "textureID" or var == "sparkID" then
		RING:Reload();
	else
		RING:Refresh();
	end
end

function _addon:getRingAlpha()
	return GCDRingDB.alpha;
end

function _addon:SetRingAlpha(alpha)
	GCDRingDB.alpha = alpha / 100;
	RING:GetFrame():SetAlpha(GCDRingDB.alpha);
end

function _addon:GetRingSize()
	local cfg = RING:GetCurrentTextureConfig();
	return cfg.outer_radius * GCDRingDB.scale;
end

function _addon:SetRingSize(size)
	local cfg = RING:GetCurrentTextureConfig();
	GCDRingDB.scale = size / cfg.outer_radius;
	RING:GetFrame():SetScale(GCDRingDB.scale);
end

function _addon:GetWatchSpellString()
	return table.concat(GCDRingDB.watchSpells, "/");
end

function _addon:SetWatchSpellStr(text)
	text = text or "";
	wipe(GCDRingDB.watchSpells);
	text:gsub("[^/]+", function(w)
		tinsert(GCDRingDB.watchSpells, w);
	end);

	addon:CheckWatchSpell();
end

function _addon:GetWatchSpell()
	return addon.watchSpell or "No match";
end

function _addon:GetBeginColor()
	return GCDRingDB.beginColor.r, GCDRingDB.beginColor.g, GCDRingDB.beginColor.b, GCDRingDB.beginColor.a;
end

function _addon:SetBeginColor(r, g, b, a)
	GCDRingDB.beginColor.r = r;
	GCDRingDB.beginColor.g = g;
	GCDRingDB.beginColor.b = b;
	GCDRingDB.beginColor.a = a;
end

function _addon:GetEndColor()
	return GCDRingDB.endColor.r, GCDRingDB.endColor.g, GCDRingDB.endColor.b, GCDRingDB.endColor.a;
end

function _addon:SetEndColor(r, g, b, a)
	GCDRingDB.endColor.r = r;
	GCDRingDB.endColor.g = g;
	GCDRingDB.endColor.b = b;
	GCDRingDB.endColor.a = a;
	if self.testMode then
		RING:SetRingColor(r, g, b, a);
	end
end

-------------------------------------------------------------------------------------------------------------------------

addon:RegisterEvent("ADDON_LOADED");

LibRingBar:RegisterRingTexture({
	name = "ring_default",
	bodyTexture = "Interface\\AddOns\\GCDRing\\Textures\\default",
	endTexture1 = "Interface\\AddOns\\GCDRing\\Textures\\slicer0",
  endTexture2 = "Interface\\AddOns\\GCDRing\\Textures\\slicer1",
  segmentsize = 128,
  outer_radius = 112,
  inner_radius = 88,
});

LibRingBar:RegisterRingTexture({
	name = "ring_thin",
	bodyTexture = "Interface\\AddOns\\GCDRing\\Textures\\thin",
	endTexture1 = "Interface\\AddOns\\GCDRing\\Textures\\slicer0",
  endTexture2 = "Interface\\AddOns\\GCDRing\\Textures\\slicer1",
  segmentsize = 128,
  outer_radius = 112,
  inner_radius = 104,
});

LibRingBar:RegisterRingTexture({
	name = "ring_broad/blend",
	bodyTexture = "Interface\\AddOns\\GCDRing\\Textures\\broad",
	endTexture1 = "Interface\\AddOns\\GCDRing\\Textures\\slicer0",
  endTexture2 = "Interface\\AddOns\\GCDRing\\Textures\\slicer1",
  segmentsize = 128,
  outer_radius = 112,
  inner_radius = 72,
});

LibRingBar:RegisterRingTexture({
	name = "ring_broad/mod",
	bodyTexture = "Interface\\AddOns\\GCDRing\\Textures\\broad",
	endTexture1 = "Interface\\AddOns\\GCDRing\\Textures\\slicer0",
  endTexture2 = "Interface\\AddOns\\GCDRing\\Textures\\slicer1",
  segmentsize = 128,
  outer_radius = 112,
  inner_radius = 72,
  blendmode = "MOD",
});

LibRingBar:RegisterRingTexture({
	name = "ring_full",
	bodyTexture = "Interface\\AddOns\\GCDRing\\Textures\\full",
	endTexture1 = "Interface\\AddOns\\GCDRing\\Textures\\slicer0",
  endTexture2 = "Interface\\AddOns\\GCDRing\\Textures\\slicer1",
  segmentsize = 128,
  outer_radius = 112,
  inner_radius = 0.001, --ops
});

LibRingBar:RegisterRingTexture({
	name = "ring_hollow",
	bodyTexture = "Interface\\AddOns\\GCDRing\\Textures\\hollow",
  segmentsize = 128,
  outer_radius = 112,
  inner_radius = 72,
});