local GearScoreElite = {};

local inspectUnit = nil;
gGSEInspectGUID = nil
gGSEInspectRealm = nil;

local inspectAchievementUnit = nil;
gGSEInspectAchievementGUID = nil
gGSEInspectAchievementRealm = nil;

local openWindowForInspectTarget = false;

BINDING_HEADER_GSE = "GearScore Elite";
BINDING_NAME_GSETARGET = "GSE Target";
BINDING_NAME_GSEDATABASE = "Player Database";

function GSE_LastRunVersionLessThan(version)
	if not gGSELastRunVersion then
		return true;
	end
	
	local major, content, minor = strsplit(".", version, 3);
	local lmajor, lcontent, lminor = strsplit(".", gGSELastRunVersion, 3);
	
	if lmajor < major then
		return true;
	elseif lmajor == major and lcontent < content then
		return true;
	elseif lmajor == major and lcontent == content and lminor < minor then
		return true;
	end
	
	return false;
end

GearScoreElite.frame = CreateFrame("Frame");

GearScoreElite.OnEvent = function(self, event, ...)
	local args = {...};
	if event == "ADDON_LOADED" and args[1] == "GearScoreElite" then
		GSE_DebugPrint("|cFFFFFF00GearScore Elite loaded.  Type |cFF00FFFF/gse|cFFFFFF00 to view detailed information about your target, or use |cFF00FFFF/gse playernamehere|cFFFFFF00 or |cFF00FFFF/gse db|cFFFFFF00 to look up information stored in your GSE database.");
		-- In version 1.0.2 we cleared any cached data.
		if (gGSEPlayerData == nil or GSE_LastRunVersionLessThan("1.0.2")) then
			gGSEPlayerData = {};
		end
		if (gGSETalentTreeData == nil or GSE_LastRunVersionLessThan("1.0.2")) then
			gGSETalentTreeData = {};
		end
		if (gGSEOptions == nil) then
			GSE_OptionsFrame_Default(GSE_OptionsFrame);
		end
		-- In version 1.1.0, the showTooltipGearScore option was split up.
		if GSE_LastRunVersionLessThan("1.1.0") then
			gGSEOptions["showTooltipItemGS"]   = gGSEOptions["showTooltipGearScore"];
			gGSEOptions["showTooltipPlayerGS"] = gGSEOptions["showTooltipGearScore"];
		end
		if not gGSEOptions["cacheTime"] then
			gGSEOptions["cacheTime"] = 60 * 60 * 24 * 7; -- This option was added in 1.0.2. We need to make sure it exists.
		end
		gGSEPlayerWindows = {};
		gGSEReusablePlayerWindows = {};
		gGSELastRunVersion = GetAddOnMetadata("GearScoreElite", "Version");
	elseif event == "PLAYER_ENTERING_WORLD" then
		GSE_UpdateInfo("player");
		GSE_UpdateTalents("player");
		GSE_UpdateInventory("player");
		GSE_UpdatePlayerWindow();
	elseif event == "UNIT_INVENTORY_CHANGED" then
		if args[1] == "player" then
			GSE_UpdateInfo(args[1]);
			GSE_UpdateTalents(args[1]);
			GSE_UpdateInventory(args[1]);
			GSE_UpdatePlayerWindow();
		end
	elseif event == "PLAYER_TALENT_UPDATE" then
		-- Necessary to prevent a bug on first run.
		if UnitGUID("player") then 
			GSE_UpdateInfo("player");
			GSE_UpdateTalents("player");
			GSE_UpdateInventory("player");
			GSE_UpdatePlayerWindow();
		end
	elseif event == "INSPECT_READY" then
		if inspectUnit and UnitGUID(inspectUnit) == args[1] then
			GSE_UpdateInfo(inspectUnit);
			GSE_UpdateTalents(inspectUnit);
			GSE_UpdateInventory(inspectUnit);
			local playerData = GSE_PlayerData(args[1], select(2, UnitName(inspectUnit)));
			if openWindowForInspectTarget then
				GSE_OpenPlayerData(playerData);
				openWindowForInspectTarget = false;
			end
			GSE_UpdatePlayerWindow(playerData);
		end
	elseif event == "INSPECT_ACHIEVEMENT_READY" then
		if inspectAchievementUnit and UnitGUID(inspectAchievementUnit) == args[1] then
			GSE_UpdateInfo(inspectAchievementUnit);
			GSE_UpdateStatistics(inspectAchievementUnit);
			local playerData = GSE_PlayerData(args[1], select(2, UnitName(inspectAchievementUnit)));
			if openWindowForInspectTarget then
				GSE_OpenPlayerData(playerData);
				openWindowForInspectTarget = false;
			end
			GSE_UpdatePlayerWindow(playerData);
			if AchievementFrameComparison then
				AchievementFrameComparison:RegisterEvent("INSPECT_ACHIEVEMENT_READY");
			end
		end
	end
	if UnitGUID("player") then -- Necessary to prevent a bug on first run.
		GSE_CleanDatabase();   -- See if the database needs to be cleaned whenever one of these things happens.
	end
end

local playerRealm = GetRealmName();

local inventorySlots = {
	"HeadSlot",
	"NeckSlot", 
	"ShoulderSlot", 
	"BackSlot", 
	"ChestSlot", 
	"ShirtSlot", 
	"TabardSlot", 
	"WristSlot", 
	"HandsSlot", 
	"WaistSlot", 
	"LegsSlot", 
	"FeetSlot", 
	"Finger0Slot", 
	"Finger1Slot", 
	"Trinket0Slot", 
	"Trinket1Slot",
	"MainHandSlot",
	"SecondaryHandSlot",
	"RangedSlot",
	"AmmoSlot",
};

local nextPlayerWindowId = 1;
local lastOpenedPlayerWindow = nil;

local lastDatabaseClean = 0;

function GSE_CleanDatabase()
	if gGSEOptions["cacheTime"] < 60 then -- This should never happen, but we'll put it here just in case.
		gGSEOptions["cacheTime"] = 60 * 15;
	end
	local cutoffTime = time() - gGSEOptions["cacheTime"];
	if cutoffTime >= lastDatabaseClean and not InCombatLockdown() then
		local safePlayerKeys = {};
		local safePlayerData = {};
		local pk = GSE_PlayerKey("player");
		if pk then
			safePlayerKeys[pk] = true;
		end
		for i = 1, 4 do
			pk = GSE_PlayerKey("party"..i);
			if pk then
				safePlayerKeys[pk] = true;
			end
		end
		for i = 1, 40 do
			pk = GSE_PlayerKey("raid"..i);
			if pk then
				safePlayerKeys[pk] = true;
			end
		end
		for name, value in pairs(gGSEPlayerWindows) do
			if value and _G[name].playerData then
				safePlayerData[_G[name].playerData] = true;
			end
		end
		for key, playerData in pairs(gGSEPlayerData) do
			if playerData["infoTime"] < cutoffTime and not safePlayerKeys[key] and not safePlayerData[playerData] then
				gGSEPlayerData[key] = nil;
			end
		end
		lastDatabaseClean = time();
	end
end

function GSE_OpenPlayerData(playerData)
	local frame = nil;
	if #gGSEReusablePlayerWindows > 0 then
		frame = tremove(gGSEReusablePlayerWindows);
		frame:SetParent(UIParent);
		frame:SetUserPlaced(false);
	elseif nextPlayerWindowId > 10 then
		GSE_DebugPrint("Close some of your existing player windows first.");
	else
		frame = CreateFrame("Frame", "GSE_PlayerFrame"..nextPlayerWindowId, UIParent, "GSE_PlayerWindowTemplate");
		nextPlayerWindowId = nextPlayerWindowId + 1;
	end
	if frame then
		frame.playerData = playerData;
		frame:ClearAllPoints();
		if lastOpenedPlayerWindow and gGSEPlayerWindows[lastOpenedPlayerWindow:GetName()] and not lastOpenedPlayerWindow:IsUserPlaced() then
			-- If the last opened window was left in its default position, stagger this one.
			local point, relativeTo, relativePoint, xOffset, yOffset = lastOpenedPlayerWindow:GetPoint(1);
			frame:SetPoint(point, relativeTo, relativePoint, xOffset + 40, yOffset - 40);
		else
			frame:SetPoint("CENTER");
		end
		ShowUIPanel(frame);
		frame:Raise();
		GSE_PlayerWindow_Refresh(frame);
		lastOpenedPlayerWindow = frame;
	end
end

function GSE_OpenDatabase(dontfocus)
	ShowUIPanel(GSE_DatabaseFrame);
	GSE_DatabaseFrame:Raise();
	if not dontfocus then
		GSE_DatabaseFrameSearchBox:SetFocus();
	end
end

function GSE_UpdatePlayerWindow(playerData)
	if not playerData then
		playerData = GSE_PlayerData(UnitGUID("player"));
	end
	for name, value in pairs(gGSEPlayerWindows) do
		if value then
			if _G[name].playerData == playerData then
				GSE_PlayerWindow_Refresh(_G[name]);
			end
		end
	end
end

-- Returns false for non-player targets.
function GSE_OpenTargetData()
	if UnitExists("target") and UnitIsPlayer("target") then
		local playerData = GSE_PlayerData(UnitGUID("target"), select(2, UnitName("target")));
		if CheckInteractDistance("target", 1) and CanInspect("target") then
			ClearInspectPlayer();
			ClearAchievementComparisonUnit();
			-- This next block is to prevent an error in Blizzard_InspectUI\InspectPaperDollFrame.lua.
			if (InspectFrame) then
				InspectFrame.unit = "player";
			end
			NotifyInspect("target");
			if AchievementFrameComparison then
				AchievementFrameComparison:RegisterEvent("INSPECT_ACHIEVEMENT_READY");
			end
			SetAchievementComparisonUnit("target");
			if not playerData then
				openWindowForInspectTarget = true;
				GSE_DebugPrint("Inspecting target . . .")
			end
		elseif not playerData then
			GSE_DebugPrint("Get within inspection range of that player to view their detailed information.")
		end
		
		if playerData then
			GSE_OpenPlayerData(playerData);
		end
		
		return true;
	end
	return false;
end

function SlashCmdList.GSE_GSE(msg, editbox)
	if not msg or msg == "" then
		if not GSE_OpenTargetData() then
			GSE_DebugPrint("Target another player to view their detailed information, or type \"/gse playernamehere\" to view player information from your stored database.");
		end
	elseif msg == "db" then
		GSE_OpenDatabase();
	else
		local playerData = GSE_PlayerDataByName(msg);
		if playerData then
			GSE_OpenPlayerData(playerData);
		else
			GSE_DebugPrint("No player with that name is stored in your GSE database.");
		end
	end
end

SLASH_GSE_GSE1 = "/gse";

GearScoreElite.frame:SetScript("OnEvent", GearScoreElite.OnEvent);
GearScoreElite.frame:RegisterEvent("ADDON_LOADED");
GearScoreElite.frame:RegisterEvent("PLAYER_ENTERING_WORLD");
GearScoreElite.frame:RegisterEvent("UNIT_INVENTORY_CHANGED");
GearScoreElite.frame:RegisterEvent("PLAYER_TALENT_UPDATE");
GearScoreElite.frame:RegisterEvent("INSPECT_READY");
GearScoreElite.frame:RegisterEvent("INSPECT_ACHIEVEMENT_READY");

function GSE_DebugPrint(msg)
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 0.8, 0);
	end
end

-- This isn't actually used by anything right now.
function GSE_StatisticId(categoryTitle, statisticTitle)
	local categoryList = GetStatisticsCategoryList();
	for c = 1, #categoryList do
		local title = GetCategoryInfo(categoryList[c]);
		if title == categoryTitle then
			local statisticCount = GetCategoryNumAchievements(categoryList[c]);
			for i = 1, statisticCount do
				local idNumber, name = GetAchievementInfo(categoryList[c], i);
				if name == statisticTitle then
					return idNumber;
				end
			end
		end
	end
	return -1;
end

function GSE_PlayerDataByName(name)
	local lowerName = name:lower();
	for playerKey, playerData in pairs(gGSEPlayerData) do
		if playerData["name"]:lower() == lowerName or (playerData["name"].."-"..playerData["realm"]):lower() == lowerName then
			return playerData;
		end
	end
end

function GSE_HookNotifyInspect(unit)
	openWindowForInspectTarget = false;
	inspectUnit = unit;
	gGSEInspectGUID = UnitGUID(unit);
	gGSEInspectRealm = GSE_Realm(select(2, UnitName(unit)));
	-- This seems to be necessary to ensure that we get accurate data later.
	for i = 1, #inventorySlots do
		local slotId, textureName = GetInventorySlotInfo(inventorySlots[i]);
		-- GetInventoryItemLink often returns nil the first time we call it.
		local itemLink = GetInventoryItemLink(unit, slotId); 
	end
end

function GSE_HookClearInspectPlayer()
	openWindowForInspectTarget = false;
	inspectUnit = nil;
	gGSEInspectGUID = nil;
	gGSEInspectRealm = nil;
end

function GSE_HookSetAchievementComparisonUnit(unit)
	openWindowForInspectTarget = false;
	inspectAchievementUnit = unit;
	gGSEInspectAchievementGUID = UnitGUID(unit);
	gGSEInspectAchievementRealm = GSE_Realm(select(2, UnitName(unit)));
end

function GSE_HookClearAchievementComparisonUnit()
	openWindowForInspectTarget = false;
	inspectAchievementUnit = nil;
	gGSEInspectAchievementGUID = nil;
	gGSEInspectAchievementRealm = nil;
end

hooksecurefunc("NotifyInspect", GSE_HookNotifyInspect);
hooksecurefunc("ClearInspectPlayer", GSE_HookClearInspectPlayer);
hooksecurefunc("SetAchievementComparisonUnit", GSE_HookSetAchievementComparisonUnit);
hooksecurefunc("ClearAchievementComparisonUnit", GSE_HookClearAchievementComparisonUnit);

function GSE_TalentPointsAtLevel(level)
	if level >= 80 then
		return 36 + (level - 80);
	elseif level >= 10 then
		return 1 + ceil((level - 10) / 2);
	end
	return 0;
end

function GSE_PlayerData(guid, realm)
	return gGSEPlayerData[guid..":"..GSE_Realm(realm):lower()];
end

function GSE_TimeIntervalString(interval)
	local days = floor(interval / (60 * 60 * 24));
	if days == 1 then
		return "1 day ago";
	elseif days > 1 then
		return days.." days ago";
	end
	local hours = floor(interval / (60 * 60));
	if hours == 1 then
		return "1 hour ago";
	elseif hours > 1 then
		return hours.." hours ago";
	end
	local minutes = floor(interval / (60));
	if minutes == 1 then
		return "1 minute ago";
	elseif minutes > 1 then
		return minutes.." minutes ago";
	end
	
	return "recently";
end

function GSE_Clamp(value, min, max)
	if value < min then
		return min;
	elseif value > max then
		return max;
	end
	return value;
end

function GSE_DirectGradientColor(value, max)
	-- Go from green to red.
	if not max then
		max = 1.0;
	end
	local ratio = GSE_Clamp(value / max, 0.0, 1.0);
	return ratio, 1.0 - ratio, 0;
end

function GSE_IndirectGradientColor(value, max)
	-- Go around the color wheel, taking the long route from green to red.
	if not max then
		max = 1.0;
	end
	local angle = GSE_Clamp(value / max, 0.0, 1.0) * 240;
	return GSE_Clamp(cos(angle + 120), 0, 1.0), GSE_Clamp(cos(angle), 0, 1.0), GSE_Clamp(cos(angle + 240), 0, 1.0);
end

function GSE_ClassColorString(class)
	return GSE_ColorString(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b, 1.0);
end

function GSE_LightenedColor(red, green, blue, alpha)
	local multiplier = 1.0 / red;
	if green > red then 
		multiplier = 1.0 / green;
	end
	if blue > red and blue > green then
		multiplier = 1.0 / blue;
	end	
	return red * multiplier, green * multiplier, blue * multiplier, alpha;
end

function GSE_LightenedColorString(red, green, blue, alpha)
	red, green, blue = GSE_LightenedColor(red, green, blue, alpha);
	if not alpha then
		return format("|cFF%02X%02X%02X", floor(red * 255), floor(green * 255), floor(blue * 255));
	end
	return format("|c%02X%02X%02X%02X", alpha, floor(red * 255), floor(green * 255), floor(blue * 255));
end

function GSE_ColorString(red, green, blue, alpha)
	if alpha == nil then
		return format("|cFF%02X%02X%02X", red * 255, green * 255, blue * 255);
	end
	return format("|c%02X%02X%02X%02X", alpha * 255, red * 255, green * 255, blue * 255);
end

function GSE_PlayerKey(unitOrGUID, realmIfGUID)
	local name, realm = UnitName(unitOrGUID);
	if name then
		return UnitGUID(unitOrGUID)..":"..GSE_Realm(realm):lower();
	end
	return unitOrGUID..":"..GSE_Realm(realmIfGUID):lower();
end

function GSE_Realm(realm)
	if realm then
		return realm;
	end
	return playerRealm;
end

function GSE_PlayerCompletion(playerData, category, mode)
	local bossesDowned = 0;
	if playerData and playerData["stats"] and playerData["stats"][category] then
		for stat, value in pairs(playerData["stats"][category]) do
			if value[mode] and value[mode] > 0 then
				bossesDowned = bossesDowned + 1;
			end
		end
	end
	return bossesDowned;
end

function GSE_PlayerCompletionPercentage(playerData, category, modes)
	-- For a player to have mastered an encounter, we'll say they need down it at least 3 times.
	local percentage = 0;
	if gGSEStatisticsInfo and gGSEStatisticsInfo[category] then
		local bosses = gGSEStatisticsInfo[category]["bosses"];
		for i = 1, #bosses do
			percentage = percentage + GSE_Clamp(GSE_PlayerBossKills(playerData, category, bosses[i], modes), 0, 3);
		end
		percentage = percentage / (#bosses * 3);
	end
	return percentage;
end

function GSE_PlayerBossKills(playerData, category, boss, modes)
	local kills = 0;
	if playerData and playerData["stats"] and playerData["stats"][category] then
		local bossKills = playerData["stats"][category][boss];
		if bossKills then
			if modes then
				for i = 1, #modes do
					if bossKills[modes[i]] then
						kills = kills + bossKills[modes[i]];
					end
				end
			else
				for mode, value in pairs(bossKills) do
					kills = kills + value;
				end
			end
		end
	end
	return kills;
end

function GSE_PlayerStatistic(playerKey, category, stat, substat)
	local statValue = 0;
	if gGSEPlayerData[playerKey] and gGSEPlayerData[playerKey]["stats"] and gGSEPlayerData[playerKey]["stats"][category] and gGSEPlayerData[playerKey]["stats"][category][stat] then
		if substat then
			if gGSEPlayerData[playerKey]["stats"][category][stat][substat] then
				statValue = gGSEPlayerData[playerKey]["stats"][category][stat][substat];
			end
		else
			for substat, value in pairs(gGSEPlayerData[playerKey]["stats"][category][stat]) do
				statValue = statValue + value;
			end
		end
	end
	return statValue;
end

function GSE_UpdateInfo(unit)
	local playerKey = GSE_PlayerKey(unit);
	if not gGSEPlayerData[playerKey] then
		gGSEPlayerData[playerKey] = {};
	end
	local name, realm = UnitName(unit);
	gGSEPlayerData[playerKey]["name"] = name;
	gGSEPlayerData[playerKey]["realm"] = GSE_Realm(realm);
	local _, classEn = UnitClass(unit);
	gGSEPlayerData[playerKey]["class"] = classEn;
	local _, raceEn = UnitRace(unit);
	gGSEPlayerData[playerKey]["race"] = raceEn;
	gGSEPlayerData[playerKey]["sex"] = UnitSex(unit);
	gGSEPlayerData[playerKey]["level"] = UnitLevel(unit);
	local race, raceEn = UnitRace(unit);
	gGSEPlayerData[playerKey]["race"] = raceEn;
	gGSEPlayerData[playerKey]["guid"] = UnitGUID(unit);
	local guildName, guildRankName = GetGuildInfo(unit);
	gGSEPlayerData[playerKey]["guildName"] = guildName;
	gGSEPlayerData[playerKey]["guildRankName"] = guildRankName;
	gGSEPlayerData[playerKey]["infoTime"] = time();
end

function GSE_UpdateStatistics(unit)
	local playerKey = GSE_PlayerKey(unit);
	if gGSEPlayerData[playerKey] then
		gGSEPlayerData[playerKey]["statsTime"] = time();
		gGSEPlayerData[playerKey]["stats"] = {};
		for key, value in pairs(gGSEStatisticsInfo) do
			gGSEPlayerData[playerKey]["stats"][key] = {};
			local keepKey = false;
			for stat, substats in pairs(value["stats"]) do
				gGSEPlayerData[playerKey]["stats"][key][stat] = {};
				local keepStat = false;
				for substat, id in pairs(substats) do
					gGSEPlayerData[playerKey]["stats"][key][stat][substat] = tonumber(GetComparisonStatistic(id));
					keepStat = true;
				end
				if not keepStat then
					gGSEPlayerData[playerKey]["stats"][key][stat] = nil;
				else
					keepKey = true;
				end
			end
			if not keepKey then
				gGSEPlayerData[playerKey]["stats"][key] = nil;
			end
		end
	end
end

function GSE_SetTalentPrereqs(class, tab, talent, ...)
	gGSETalentTreeData[class][tab][talent]["prereqs"] = {};
	for i = 1, select("#", ...), 4 do
		local tier, column, isLearnable, isPreviewLearnable = select(i, ...);
		tinsert(gGSETalentTreeData[class][tab][talent]["prereqs"], {["tier"] = tier, ["column"] = column});
	end
end

function GSE_UpdateTalents(unit)
	local inspectUnit = not UnitIsUnit(unit, "player");
	local playerKey = GSE_PlayerKey(unit);
	if gGSEPlayerData[playerKey] then
		gGSEPlayerData[playerKey]["talentsTime"] = time();
		gGSEPlayerData[playerKey]["talents"] = {};
		local talentGroup = GetActiveTalentGroup(inspectUnit);
		gGSEPlayerData[playerKey]["talentGroup"] = talentGroup;
		gGSEPlayerData[playerKey]["talentBuild"] = select(2, GetBuildInfo());
		if not gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]] then
			gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]] = {};
		end
		gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]]["build"] = select(2, GetBuildInfo());
		local numberOfTabs = GetNumTalentTabs(inspectUnit);
		for talentGroup = 1, 2 do
			gGSEPlayerData[playerKey]["talents"][talentGroup] = {};
			local primaryTab = nil;
			local primaryTabPoints = 0;
			for tab = 1, numberOfTabs do
				local _, tabName, description, tabIconTexture, pointsSpent, background, _, isUnlocked = GetTalentTabInfo(tab, inspectUnit, false, talentGroup);
				if pointsSpent > primaryTabPoints then
					primaryTab = tab;
					primaryTabPoints = pointsSpent;
				end
				gGSEPlayerData[playerKey]["talents"][talentGroup][tab] = {};
				gGSEPlayerData[playerKey]["talents"][talentGroup][tab]["pointsSpent"] = pointsSpent;
				gGSEPlayerData[playerKey]["talents"][talentGroup][tab]["isUnlocked"] = isUnlocked;
				if not gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab] then
					gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab] = {};
				end
				gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab]["name"] = tabName;
				gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab]["description"] = description;
				gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab]["iconTexture"] = tabIconTexture;
				gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab]["background"] = background;
				local role1, role2 = GetTalentTreeRoles(tab, inspectUnit);
				gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab]["role1"] = role1;
				gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab]["role2"] = role2;
				local numberOfTalents = GetNumTalents(tab, inspectUnit);
				for i = 1, numberOfTalents do
					local talentName, talentIconTexture, tier, column, rank, maxRank, _, _, _, isExceptional = GetTalentInfo(tab, i, inspectUnit, false, talentGroup);
					if rank > 0 then
						gGSEPlayerData[playerKey]["talents"][talentGroup][tab][i] = rank;
					end
					if not gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab][i] then
						gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab][i] = {};
					end
					GSE_SetTalentPrereqs(gGSEPlayerData[playerKey]["class"], tab, i, GetTalentPrereqs(tab, i, inspectUnit, false, talentGroup));
					gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab][i][rank] = GetTalentLink(tab, i, inspectUnit, false, talentGroup);
					gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab][i]["name"] = talentName;
					gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab][i]["iconTexture"] = talentIconTexture;
					gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab][i]["tier"] = tier;
					gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab][i]["column"] = column;
					gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab][i]["maxRank"] = maxRank;
					gGSETalentTreeData[gGSEPlayerData[playerKey]["class"]][tab][i]["isExceptional"] = isExceptional;
				end
			end
			gGSEPlayerData[playerKey]["talents"][talentGroup]["primaryTab"] = primaryTab;
		end
	end
end

function GSE_UpdateInventory(unit)
	GSE_UpdateInfo(unit);
	local playerKey = GSE_PlayerKey(unit);
	local inspectUnit = not UnitIsUnit(unit, "player");
	if gGSEPlayerData[playerKey] then
		if not gGSEPlayerData[playerKey]["inventory"] then
			gGSEPlayerData[playerKey]["inventory"] = {};
		end
		if not gGSEPlayerData[playerKey]["inventoryTime"] then
			gGSEPlayerData[playerKey]["inventoryTime"] = {};
		end
		local talentGroup = GetActiveTalentGroup(inspectUnit);
		gGSEPlayerData[playerKey]["inventory"][talentGroup] = {};
		gGSEPlayerData[playerKey]["inventoryTime"][talentGroup] = time();
		for i = 1, #inventorySlots do
			local slotId, textureName = GetInventorySlotInfo(inventorySlots[i]);
			gGSEPlayerData[playerKey]["inventory"][talentGroup][inventorySlots[i]] = GetInventoryItemLink(unit, slotId);
		end
	end
end

function GSE_TotalGemSockets(item)
	-- Doesn't include the eternal belt buckle's socket.
	local itemStats = GetItemStats(item);
	local totalGemSockets = 0;
	if itemStats then
		if itemStats["EMPTY_SOCKET_YELLOW"] then
			totalGemSockets = totalGemSockets + itemStats["EMPTY_SOCKET_YELLOW"];
		end
		if itemStats["EMPTY_SOCKET_RED"] then
			totalGemSockets = totalGemSockets + itemStats["EMPTY_SOCKET_RED"];
		end
		if itemStats["EMPTY_SOCKET_BLUE"] then
			totalGemSockets = totalGemSockets + itemStats["EMPTY_SOCKET_BLUE"];
		end
		if itemStats["EMPTY_SOCKET_META"] then
			totalGemSockets = totalGemSockets + itemStats["EMPTY_SOCKET_META"];
		end
		if itemStats["EMPTY_SOCKET_PRISMATIC"] then
			totalGemSockets = totalGemSockets + itemStats["EMPTY_SOCKET_PRISMATIC"];
		end
	end
	return totalGemSockets;
end

function GSE_TotalGems(item)
	local _, _, _, jewel1, jewel2, jewel3, jewel4 = strsplit(":", item);
	local totalGems = 0;
	if jewel1 ~= "0" then
		totalGems = totalGems + 1;
	end			
	if jewel2 ~= "0" then
		totalGems = totalGems + 1;
	end	
	if jewel3 ~= "0" then
		totalGems = totalGems + 1;
	end	
	if jewel4 ~= "0" then
		totalGems = totalGems + 1;
	end	
	return totalGems;
end