local tooltipGearScoreLine = nil;
local tooltipTalentsLine = nil;
local tooltipStatisticsLine1 = nil;
local tooltipStatisticsLine2 = nil;

local isInspectingTooltipUnit = false;
local tooltipGUID = nil;
local tooltipRealm = nil;

function GSE_UpdateTooltip(guid, realm)
	local playerData = GSE_PlayerData(guid, realm);
	if gGSEOptions["showTooltipPlayerGS"] and playerData and playerData["inventory"] and playerData["talentGroup"] and playerData["inventory"][playerData["talentGroup"]] then
		if tooltipGearScoreLine == nil then
			tooltipGearScoreLine = GSE_AddTooltipLine(GameTooltip); 
		end
		local talentGroup = playerData["talentGroup"];
		local gearScore = GSE_GearScore(playerData, talentGroup);
		local interval = time() - playerData["inventoryTime"][talentGroup];
		local gsColor = GSE_LightenedColorString(GSE_GearScoreColor(gearScore));
		local tIColor = GSE_LightenedColorString(GSE_DirectGradientColor(interval, 60 * 60 * 24 * 7));
		tooltipGearScoreLine:SetFormattedText("|cFFF0F070GearScore: %s%d %s(updated %s)", gsColor, gearScore, tIColor, GSE_TimeIntervalString(interval));
	end
	if gGSEOptions["showTooltipTalents"] and playerData and playerData["talents"] then
		if tooltipTalentsLine == nil then
			tooltipTalentsLine = GSE_AddTooltipLine(GameTooltip); 
		end
		local specNames = {"Undecided", "Undecided"};
		local specColors = {"FFA0A0A0", "FFA0A0A0"};
		for spec = 1, 2 do
			local pointsSpent = playerData["talents"][spec][1]["pointsSpent"] + playerData["talents"][spec][2]["pointsSpent"] + playerData["talents"][spec][3]["pointsSpent"];
			if playerData["talents"][spec]["primaryTab"] then
				specNames[spec] = gGSETalentTreeData[playerData["class"]][playerData["talents"][spec]["primaryTab"]]["name"];
			end
			if playerData["talentGroup"] == spec then
				if pointsSpent < GSE_TalentPointsAtLevel(playerData["level"]) then
					specColors[spec] = "FFFF0000";
				else
					specColors[spec] = "FFFFFFFF";
				end
			elseif playerData["level"] >= 40 and pointsSpent < GSE_TalentPointsAtLevel(playerData["level"]) then
				specColors[spec] = "FFA00000";
			end
		end
		tooltipTalentsLine:SetFormattedText("|c%s%s: %d %d %d|cFFA0A0A0, |c%s%s: %d %d %d", specColors[1], specNames[1], playerData["talents"][1][1]["pointsSpent"], playerData["talents"][1][2]["pointsSpent"], playerData["talents"][1][3]["pointsSpent"], specColors[2], specNames[2], playerData["talents"][2][1]["pointsSpent"], playerData["talents"][2][2]["pointsSpent"], playerData["talents"][2][3]["pointsSpent"]);
	end
	if gGSEOptions["showTooltipExperience"] and playerData and playerData["stats"] and playerData["level"] >= 80 then
		if tooltipStatisticsLine1 == nil then
			tooltipStatisticsLine1 = GSE_AddTooltipLine(GameTooltip); 
		end
		if tooltipStatisticsLine2 == nil then
			tooltipStatisticsLine2 = GSE_AddTooltipLine(GameTooltip); 
		end
		tooltipStatisticsLine1:SetFormattedText(gGSEStatisticsInfo["rs"].getSummaryLine(playerData));
		tooltipStatisticsLine2:SetFormattedText(gGSEStatisticsInfo["icc"].getSummaryLine(playerData));
	end
	GameTooltip:Show();
end

function GSE_AddTooltipLine(tooltip)
	tooltip:AddLine(" ", 0, 0, 0);
	return _G[tooltip:GetName().."TextLeft"..tooltip:NumLines()];
end

function GSE_Tooltips_HookGSE_UpdateSomething(unit)
	if GameTooltip:IsUnit(unit) then
		-- Update the tooltip.
		local name, realm = UnitName(unit);
		GSE_UpdateTooltip(UnitGUID(unit), realm);
	end
end

hooksecurefunc("GSE_UpdateInventory", GSE_Tooltips_HookGSE_UpdateSomething);
hooksecurefunc("GSE_UpdateTalents", GSE_Tooltips_HookGSE_UpdateSomething);
hooksecurefunc("GSE_UpdateStatistics", GSE_Tooltips_HookGSE_UpdateSomething);
	
function GSE_HookOnTooltipSetUnit(self)
	tooltipGearScoreLine = nil;
	tooltipTalentsLine = nil;
	tooltipStatisticsLine1 = nil;
	tooltipStatisticsLine2 = nil;
	GSE_ClearTooltipInspection();
	local _, unit = GameTooltip:GetUnit();
	if unit and UnitIsPlayer(unit) then
		local name, realm = UnitName(unit);
		if not InCombatLockdown() and CheckInteractDistance(unit, 1) and CanInspect(unit) then
			-- If possible, inspect the unit and update the tooltip based on their current gear.
			if gGSEOptions["showTooltipPlayerGS"] then
				tooltipGearScoreLine = GSE_AddTooltipLine(GameTooltip); 
				tooltipGearScoreLine:SetText("|cFFA0A0A0Inspecting . . .");
			end
			if gGSEOptions["showTooltipTalents"] then
				tooltipTalentsLine = GSE_AddTooltipLine(GameTooltip); 
				if not gGSEOptions["showTooltipGearScore"] then
					tooltipTalentsLine:SetText("|cFFA0A0A0Inspecting . . .");
				end
			end
			if gGSEOptions["showTooltipExperience"] and UnitLevel(unit) >= 80 then
				tooltipStatisticsLine1 = GSE_AddTooltipLine(GameTooltip); 
				tooltipStatisticsLine1:SetText("|cFFA0A0A0Loading stats . . .");
				tooltipStatisticsLine2 = GSE_AddTooltipLine(GameTooltip); 
			end
			GSE_SetTooltipInspection(unit);
		end
		GSE_UpdateTooltip(UnitGUID(unit), realm);
	end
end

function GSE_HookOnTooltipSetItem(self)
	local _, item = self:GetItem();
	local gearScore = GSE_ItemGearScore(item);
	if gGSEOptions["showTooltipItemGS"] and gearScore > 0 then
		local newLine = GSE_AddTooltipLine(self); 
		local gsColor = GSE_LightenedColorString(GSE_ItemGearScoreColor(item));
		local _, classEn = UnitClass("player");
		local _, _, _, _, _, _, _, _, itemEquipLoc = GetItemInfo(item);
		if classEn == "WARRIOR" and itemEquipLoc == "INVTYPE_2HWEAPON" then
			newLine:SetFormattedText("|cFFF0F070GearScore: %s%d (%d)", gsColor, gearScore, gearScore * 0.5);
		elseif classEn == "HUNTER" and (itemEquipLoc == "INVTYPE_WEAPON" or itemEquipLoc == "INVTYPE_2HWEAPON" or itemEquipLoc == "INVTYPE_WEAPONMAINHAND" or itemEquipLoc == "INVTYPE_WEAPONOFFHAND") then
			newLine:SetFormattedText("|cFFF0F070GearScore: %s%d (%d)", gsColor, gearScore, gearScore * 0.3164);
		elseif classEn == "HUNTER" and (itemEquipLoc == "INVTYPE_RANGED") then
			newLine:SetFormattedText("|cFFF0F070GearScore: %s%d (%d)", gsColor, gearScore, gearScore * 5.3224);
		else
			newLine:SetFormattedText("|cFFF0F070GearScore: %s%d", gsColor, gearScore);
		end
	end
end

function GSE_SetTooltipInspection(unit)
	ClearInspectPlayer();
	ClearAchievementComparisonUnit();
	isInspectingTooltipUnit = true;
	tooltipGUID = UnitGUID(unit);
	tooltipRealm = GSE_Realm(select(2, UnitName(unit)));
	-- This next block is to prevent an error in Blizzard_InspectUI\InspectPaperDollFrame.lua.
	if (InspectFrame) then
		InspectFrame.unit = "player";
	end
	NotifyInspect(unit);
	if AchievementFrameComparison then
		AchievementFrameComparison:RegisterEvent("INSPECT_ACHIEVEMENT_READY");
	end
	SetAchievementComparisonUnit(unit);
end

function GSE_ClearTooltipInspection()
	if isInspectingTooltipUnit then
		if gGSEInspectGUID == tooltipGUID and gGSEInspectRealm == tooltipRealm then
			ClearInspectPlayer();
		end
		if gGSEInspectAchievementGUID == tooltipGUID and gGSEInspectAchievementRealm == tooltipRealm then
			ClearAchievementComparisonUnit();
		end
		isInspectingTooltipUnit = false;
	end
end

function GSE_HookOnTooltipCleared(self)
	GSE_ClearTooltipInspection();
end

ItemRefTooltip:HookScript("OnTooltipSetItem", GSE_HookOnTooltipSetItem);

GameTooltip:HookScript("OnTooltipSetUnit", GSE_HookOnTooltipSetUnit);
GameTooltip:HookScript("OnTooltipSetItem", GSE_HookOnTooltipSetItem);
GameTooltip:HookScript("OnTooltipCleared", GSE_HookOnTooltipCleared);

local shoppingTooltip1, shoppingTooltip2, shoppingTooltip3 = unpack(GameTooltip.shoppingTooltips);

shoppingTooltip1:HookScript("OnTooltipSetItem", GSE_HookOnTooltipSetItem);
shoppingTooltip2:HookScript("OnTooltipSetItem", GSE_HookOnTooltipSetItem);
shoppingTooltip3:HookScript("OnTooltipSetItem", GSE_HookOnTooltipSetItem);

local shoppingTooltip1, shoppingTooltip2, shoppingTooltip3 = unpack(ItemRefTooltip.shoppingTooltips);

shoppingTooltip1:HookScript("OnTooltipSetItem", GSE_HookOnTooltipSetItem);
shoppingTooltip2:HookScript("OnTooltipSetItem", GSE_HookOnTooltipSetItem);
shoppingTooltip3:HookScript("OnTooltipSetItem", GSE_HookOnTooltipSetItem);
