-- Author:			Naberius (Mysterio of Perenolde-US)
-- Description:		Alerts you of missing buffs

local useMOD = true			-- if you want to use this tool leave true
if not useMOD then return end

local party = true			-- do the buff checking not only in a raid but also a party
local waiter = 5			-- threshold timer for your food, flask & wep enchant buff in minutes
local ignore_bgs = true			-- ignore buff checks in a battleground/arena/voa/tol barad

local reminder = CreateFrame('Frame')
local pClass = select(2, UnitClass('player'))


local function grouped()

   if GetNumPartyMembers() > 0 then return true end
   return false
end

local function inRaid(class)

   local gt = 'none'

   if (GetNumRaidMembers() > 0) then
      totalMems = GetNumRaidMembers()
      gt = 'raid'
   elseif (GetNumPartyMembers() > 0) then
      totalMems = GetNumPartyMembers()
      gt = 'party'
   end

   if (totalMems) then
      for i = 1, totalMems do
         local unit = gt..i
         if not UnitIsPlayer('unit') and select(2, UnitClass(unit)) == class then return true end
      end
   end

   return false
end

local function classCount(class)

   local cc = 0

   if (GetNumRaidMembers() > 0) then
      totalMems = GetNumRaidMembers()
      gt = 'raid'
   elseif (GetNumPartyMembers() > 0) then
      totalMems = GetNumPartyMembers()
      gt = 'party'
   end

   if (totalMems) then
      for i = 1, totalMems do
         local unit = gt..i
         if not UnitIsPlayer('unit') and select(2, UnitClass(unit)) == class then cc = cc + 1 end
      end
   end

   return cc
end

local function buffCheck(name)

   return UnitAura("player", name, nil, "HELPFUL")
end

local function food(name)

   local name, _, _, _, _, _, expirationTime = UnitAura("player", "Well Fed", nil, "HELPFUL")
	
   if name then return true end
   return false
end

local function foodTimer(name)

   local name, _, _, _, _, _, expirationTime = UnitAura("player", "Well Fed", nil, "HELPFUL")
	
   if name then return expirationTime - GetTime() <= (waiter*60) end
end

local function flask()

   for i = 1, BUFF_MAX_DISPLAY do
      local name, _, _, _, _, _, expirationTime = UnitAura("player", i, "HELPFUL|CANCELABLE")
	
      if name then
         if (name:sub(1, 5) == "Flask") then return true end			
      else return false end
   end
end

local function flaskTimer()

   for i = 1, BUFF_MAX_DISPLAY do
      local name, _, _, _, _, _, expirationTime = UnitAura("player", i, "HELPFUL|CANCELABLE")
	
      if name then
         if (name:sub(1, 5) == "Flask") then return expirationTime - GetTime() <= (waiter*60) end			
      else return false end
   end
end

local function hasValidWeapon(offHand)

   local weaponClass = select(1, GetAuctionItemClasses())
   local fishingPoleClass = select(17, GetAuctionItemSubClasses(1))
   local link = GetInventoryItemLink("player", offHand and 17 or 16)
		
   if link then
      local class, subClass = select(6, GetItemInfo(link))
      return class == weaponClass and subClass ~= fishingPoleClass
   end
end

local function enchantTimer(offHand)

   local hasMainHandEnchant, mainHandExpiration, _, hasOffHandEnchant, offHandExpiration = GetWeaponEnchantInfo()
		
   if offHand then
      return hasOffHandEnchant and (offHandExpiration / 1000) or -1
   else
      return hasMainHandEnchant and (mainHandExpiration / 1000) or -1
   end
end

local function hasTalent(tabIndex, talentIndex, rankRequired)

   return select(5, GetTalentInfo(tabIndex, talentIndex)) >= (rankRequired or 1)
end

local function bgORarena()

   local zone = GetZoneText()

   if (zone == "Warsong Gulch" or zone == "Battle for Gilneas" or zone == "Twin Peaks" or zone == "Arathi Basin" or 
       zone == "Eye of the Storm" or zone == "Strand of the Ancients" or zone == "Wintergrasp" or zone == "Alterac Valley" 
       or zone == "Isle of Conquest") then
         return true
   elseif (zone == "The Circle of Blood" or zone == "Dalaran Sewers" or zone == "Ruins of Lordaeron" or 
       zone == "The Ring of Trials" or zone == "The Ring of Valor") then
         return true
   elseif (zone == "Vault of Archavon" or zone == "Tol Barad") then
         return true
   end

   return false
end

local gb, sb = {}, {}

local function bufforama()

   if not grouped then return end
   if not party and grouped then return end
   if ignore_bgs and bgORarena then return end

   -- group buff checks

   if inRaid("DRUID") then
      if not buffCheck("Mark of the Wild") then table.insert(gb, "Mark of the Wild") end
   end

   if inRaid("PALADIN") then
      if not inRaid("DRUID") then
         if not buffCheck("Blessing of Kings") then table.insert(gb, "Blessing of Kings") end
      else
         if not buffCheck("Blessing of Might") then table.insert(gb, "Blessing of Might") end
      end
   end

   if inRaid("MAGE") then
      if not inRaid("PALADIN") then
         if not buffCheck("Arcane Brilliance") or not buffCheck("Dalaran Brilliance") then table.insert(gb, "Arcane Brilliance") end
      end
   end

   if inRaid("PRIEST") then
      if not buffCheck("Power Word: Fortitude") then table.insert(gb, "Power Word: Fortitude") end
   end

   if inRaid("WARRIOR") then
      if not inRaid("PRIEST") and not inRaid("WARLOCK") then
         if not buffCheck("Commanding Shout") then table.insert(gb, "Commanding Shout") end
      end
   end

   if inRaid("WARLOCK") then
      if not inRaid("PRIEST") and not inRaid("WARRIOR") then
         if not buffCheck("Blood Pact") then table.insert(gb, "Blood Pact") end
      end
   end

   -- self buff checks

   if not food("Well Fed") or foodTimer("Well Fed") then table.insert(sb, "Food") end
   if not flask or flaskTimer then table.insert(sb, "Flask") end

   if pClass == "HUNTER" then
      if not buffCheck("Aspect of the Hawk") and not buffCheck("Aspect of the Cheetah") and not buffCheck("Aspect of the Pack") and 
         not buffCheck("Aspect of the Wild") and not buffCheck("Aspect of the Fox") then table.insert(sb, "Aspect") end
      if not (IsMounted() or PetHasActionBar()) then table.insert(sb, "Pet") end
   end

   local group_output = table.concat(gb, ", ")
   local self_output = table.concat(sb, ", ")

   if #gb > 0 or #sb > 0 then print('|cffcd0000Missing the following buffs!|r') end
   if #gb > 0 then print('|cffcd0000Group:|r '..group_output) end
   if #sb > 0 then print('|cffcd0000Self:|r '..self_output) end
   wipe(gb)
   wipe(sb)
end

reminder:RegisterEvent('READY_CHECK')
reminder:SetScript('OnEvent', bufforama)