local L = LibStub("AceLocale-3.0"):GetLocale("SkadaAvoidanceMitigation", false)

local Skada = Skada

local mod = Skada:NewModule(L["Avoidance and mitigation"])
local reduction_breakdown_mod = Skada:NewModule(L["incoming physical damage breakdown"])

local function capitalize(str)
  local retval = ""
  str = tostring(str)
  retval = str:sub(1,1):upper() .. str:sub(2):lower()
  return retval
end

local function log_attack_result(set, dmg)
  -- Get the player.
  local player = Skada:get_player(set, dmg.playerid, dmg.playername)
  if player and dmg.result then
    -- Add to player breakdown
    if not player.swing_damage_breakdown[dmg.result] then
      player.swing_damage_breakdown[dmg.result] = 0
    end

    player.swing_damage_breakdown[dmg.result] = player.swing_damage_breakdown[dmg.result] + 1
    player.total_events = player.total_events + 1
  end
end
 
local dmg = {}

local function Damage(amt_blocked, crit_flag, crush_flag, glance_flag, dmg)
  if amt_blocked and amt_blocked > 0 then
    dmg.result = L["BLOCK"]
  elseif crit_flag then
    dmg.result = L["CRIT"]
  elseif crush_flag then
    dmg.result = L["CRUSH"]
  else
    dmg.result = L["HIT"]
  end

  log_attack_result(Skada.current, dmg)
  log_attack_result(Skada.total, dmg)
end

local function SwingDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
  -- White melee.
  local amt, amt_overkill, spell_school, amt_resisted, amt_blocked, amt_absorbed, crit_flag, glancing_flag, crush_flag = ...

  dmg.playerid = dstGUID
  dmg.playername = dstName
  dmg.spellid = 6603
  dmg.spellname = L["Auto Attack"]

  Damage(amt_blocked, crit_flag, crush_flag, glancing_flag, dmg)
end

local function SpellDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
  local spellId, spellName, spellSchool, amt, amt_overkill, spell_school, amt_resisted, amt_blocked, amt_absorbed, crit_flag, glancing_flag, crush_flag = ...

  -- Ignore all spells that don't deal physical damage
  if 0 == (bit.band(spellSchool, 0x1)) then
    return
  end

  dmg.playerid = dstGUID
  dmg.playername = dstName
  dmg.spellid = spellId
  dmg.spellname = spellName

  Damage(amt_blocked, crit_flag, crush_flag, glancing_flag, dmg)
end

local function Missed(miss_type, dmg)
  -- Treat absorbed attacks as hits, since it has to 
  if  "ABSORB" == miss_type then
    -- Clarify that 100% of the  damage was absorbed (meaning we can't
    -- tell if it was blocked, or otherwise partially reduced)
    dmg.result = L["FULL ABSORB"]
  else
    dmg.result = L[miss_type:upper()]
  end

  log_attack_result(Skada.current, dmg)
  log_attack_result(Skada.total, dmg)
end

local function SwingMissed(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
  local miss_type, amount_missed = ...

  dmg.playerid = dstGUID
  dmg.playername = dstName
  dmg.spellid = 6603
  dmg.spellname = L["Auto Attack"]

  Missed(miss_type, dmg)
end

local function SpellMissed(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
  local spellId, spellName, spellSchool, miss_type, amount_missed = ...

  if 0 == (bit.band(spellSchool, 0x1)) then
    return
  end

  dmg.playerid = dstGUID
  dmg.playername = dstName
  dmg.spellid = spellId
  dmg.spellname = spellName

  Missed(miss_type, dmg)
end

-- Update the data set displayed
function mod:Update(win, set)
  local max = 0
  local avoid_count = 0
    
  local nr = 1
  for i, player in ipairs(set.players) do

    -- next returns nil if the table is empty
    if next(player.swing_damage_breakdown) then
      local d = win.dataset[nr] or {}
      win.dataset[nr] = d

      -- The  main data  to pay  attention to:  how much  was incoming
      -- damage reduced by, on average: more is better
      avoid_count = ((player.swing_damage_breakdown[L["BLOCK"]] or 0) +
                     (player.swing_damage_breakdown[L["DODGE"]] or 0) +
                     (player.swing_damage_breakdown[L["MISS"]] or 0) +
                     (player.swing_damage_breakdown[L["PARRY"]] or 0))
      total = 0

      for mtype, count in pairs(player.swing_damage_breakdown) do
        total = total + count
      end

      d.label = player.name
      d.value = (avoid_count * 100.0 / total)
      d.valuetext = ("%02.1f%% (%d)"):format(d.value, avoid_count)
      d.id = player.id
      d.class = player.class

      if d.value > max then
        max = d.value
      end
      nr = nr + 1
    end
  end
    
  win.metadata.maxvalue = max
end

function mod:OnEnable()
  mod.metadata                     = {showspots = true, click1 = reduction_breakdown_mod}
  reduction_breakdown_mod.metadata = {}

  Skada:RegisterForCL(SwingDamage, 'SWING_DAMAGE', {dst_is_interesting_nopets = true})
  Skada:RegisterForCL(SwingMissed, 'SWING_MISSED', {dst_is_interesting_nopets = true})
  Skada:RegisterForCL(SpellDamage, 'SPELL_DAMAGE', {dst_is_interesting_nopets = true})
  Skada:RegisterForCL(SpellMissed, 'SPELL_MISSED', {dst_is_interesting_nopets = true})

  Skada:AddMode(self)
end

function mod:OnDisable()
  Skada:RemoveMode(self)
end

-- Called by Skada when a new player is added to a set.
function mod:AddPlayerAttributes(player)
  if not player.swing_damage_breakdown then
    player.swing_damage_breakdown = {}
    player.total_events = 0
  end
end


function reduction_breakdown_mod:Enter(win, id, label)
  local player = Skada:find_player(win:get_selected_set(), id)
  reduction_breakdown_mod.playerid = id
  reduction_breakdown_mod.title = player.name..L["'s "]..L["incoming physical breakdown"]
end

-- Player view showing breakdown of damage reduction
function reduction_breakdown_mod:Update(win, set)
  local player = Skada:find_player(set, self.playerid)
  local max = 0
	
  -- If we reset we have no data.
  if player then
    
    local nr = 1
    for mtype, count in pairs(player.swing_damage_breakdown) do
      local d = win.dataset[nr] or {}
      win.dataset[nr] = d

      d.label = mtype
      d.id = mtype
      d.value = (count / player.total_events) * 100
      d.valuetext = ("%d"):format(count)..(" (%02.1f%%)"):format(d.value)

      if d.value > max then
        max = d.value
      end

      nr = nr + 1
    end
  end

  win.metadata.maxvalue = max
end
