local nibSpellAlertConfig = LibStub("AceAddon-3.0"):GetAddon("nibSpellAlertConfig");
local db;

local table_Strata = {
	"BACKGROUND",
	"LOW",
	"MEDIUM",
	"HIGH",
	"DIALOG",
	"TOOLTIP",
};

local ClassAlerts_Sides = {
	"None",
	"Killing Machine",
	"Sudden Doom",
	"Clearcasting",
	"Focus Fire",
	"Arcane Missiles",
	"Brain Freeze",
	"Hot Streak",
	"Art of War",
	"Daybreak",
	"Grand Crusader",
	"Surge of Light",
	"Nightfall",
	"Empowered Imp",
	"Molten Core",
	"Bloodsurge",
	"Sudden Death",
	"Sword and Board",
};

local ClassAlerts_Top = {
	"None",
	"Dark Transformation",
	"Freezing Fog",
	"Fury of Stormrage",
	"Shooting Stars",
	"Lock and Load",
	"Ready, Set, Aim",
	"Fingers of Frost",
	"Impact",
	"Hand of Light",
	"Maelstrom Weapon",
	"Backlash",
};

local ClassAlerts_TopRight = {
	"None",
	"Eclipse (Solar)",
};

local ClassAlerts_TopLeft = {
	"None",
	"Eclipse (Lunar)",
};

local function ValidateOffset(value)
	val = tonumber(value);
	if val == nil then val = 0; end;
	if val < -5000 then val = 5000 elseif val > 5000 then val = 5000; end;
	return val;
end

-- Return the Options table
local options = nil;
local function GetOptions()
	if not options then
		options = {
			name = "nibSpellAlertConfig",
			handler = nibSpellAlertConfig,
			type = "group",
			childGroups = "tab",
			args = {
			-- Config Guide
				configguide = {
					name = "Config Guide",
					type = "group",
					order = 10,
					args = {
						enabled = {
							name = "Enabled",
							type = "toggle",
							get = function() return db.configguide.enabled end,
							set = function(info, value) 
								db.configguide.enabled = value;
								nibSpellAlertConfig:ShowPreviewArea();
							end,
							order = 10,
						},
						displayspells = {
							name = "Config Guide display",
							type = "group",
							inline = true,
							order = 20,
							args = {
								sides = {
									name = "Sides",
									type = "select",
									get = function(info) 
										for k_ts,v_ts in pairs(ClassAlerts_Sides) do
											if v_ts == db.configguide.spells.sides then return k_ts end
										end
									end,
									set = function(info, value)
										nibSpellAlertConfig:HidePreviewAlerts("Sides");
										db.configguide.spells.sides = ClassAlerts_Sides[value];	
									end,
									values = ClassAlerts_Sides,
									order = 10,
								},
								top = {
									name = "Top",
									type = "select",
									get = function(info) 
										for k_ts,v_ts in pairs(ClassAlerts_Top) do
											if v_ts == db.configguide.spells.top then return k_ts end
										end
									end,
									set = function(info, value)
										nibSpellAlertConfig:HidePreviewAlerts("Top");
										db.configguide.spells.top = ClassAlerts_Top[value];
									end,
									values = ClassAlerts_Top,
									order = 20,
								},
								topright = {
									name = "Top Right",
									type = "select",
									get = function(info) 
										for k_ts,v_ts in pairs(ClassAlerts_TopRight) do
											if v_ts == db.configguide.spells.topright then return k_ts end
										end
									end,
									set = function(info, value)
										nibSpellAlertConfig:HidePreviewAlerts("TopRight");
										db.configguide.spells.topright = ClassAlerts_TopRight[value];
									end,
									values = ClassAlerts_TopRight,
									order = 30,
								},
								topleft = {
									name = "Top Left",
									type = "select",
									get = function(info) 
										for k_ts,v_ts in pairs(ClassAlerts_TopLeft) do
											if v_ts == db.configguide.spells.topleft then return k_ts end
										end
									end,
									set = function(info, value)
										nibSpellAlertConfig:HidePreviewAlerts("TopLeft");
										db.configguide.spells.topleft = ClassAlerts_TopLeft[value];
									end,
									values = ClassAlerts_TopLeft,
									order = 40,
								},
							},
						},						
					},
				},
			-- Position
				position = {
					name = "Position",
					type = "group",
					order = 30,
					args = {
						offsetarea = {
							name = "Position",
							type = "group",
							inline = true,
							order = 10,
							args = {
								xofs = {
									type = "input",
									name = "X Offset",
									width = "half",
									order = 10,
									get = function(info) return tostring(db.position.x) end,
									set = function(info, value)
										value = ValidateOffset(value)
										db.position.x = value
										nibSpellAlertConfig:UpdatePosition()
									end,
								},
								yofs = {
									type = "input",
									name = "Y Offset",
									width = "half",
									order = 20,
									get = function(info) return tostring(db.position.y) end,
									set = function(info, value)
										value = ValidateOffset(value)
										db.position.y = value
										nibSpellAlertConfig:UpdatePosition()
									end,
								},
							},
						},
						framelevelarea = {
							name = "Frame Level",
							type = "group",
							inline = true,
							order = 20,
							args = {
								framestrata = {
									type = "select",
									name = "Frame Strata",
									get = function(info) 
										for k_ts,v_ts in pairs(table_Strata) do
											if v_ts == db.framelevel.strata then return k_ts end
										end
									end,
									set = function(info, value)
										db.framelevel.strata = table_Strata[value]
										nibSpellAlertConfig:UpdatePosition()
									end,
									style = "dropdown",
									width = nil,
									values = table_Strata,
									order = 10,
								},
								level = {
									type = "range",
									name = "Frame Level",
									min = 1, max = 50, step = 1,
									get = function(info) return db.framelevel.level end,
									set = function(info, value) 
										db.framelevel.level = value
										nibSpellAlertConfig:UpdatePosition()
									end,
									order = 20,
								},
							},
						},
					},
				},
			-- Size
				size = {
					type = "group",
					name = "Size",
					order = 40,
					args = {
						width = {
							type = "input",
							name = "Width",
							width = "half",
							order = 10,
							get = function(info) return tostring(db.size.width) end,
							set = function(info, value)
								value = ValidateOffset(value)
								db.size.width = value
								nibSpellAlertConfig:UpdatePosition()
							end,
						},
						height = {
							type = "input",
							name = "Height",
							width = "half",
							order = 20,
							get = function(info) return tostring(db.size.height) end,
							set = function(info, value)
								value = ValidateOffset(value)
								db.size.height = value
								nibSpellAlertConfig:UpdatePosition()
							end,
						},
						scale = {
							type = "range",
							name = "Scale",
							min = 0.2, max = 2, step = 0.01,
							isPercent = true,
							get = function(info) return db.size.scale end,
							set = function(info, value) 
								db.size.scale = value
								nibSpellAlertConfig:UpdatePosition()
							end,
							order = 30,
						},
					},
				},
			-- Appearance
				appearance = {
					name = "Appearance",
					type = "group",
					order = 50,
					args = {
						opacity = {
							type = "range",
							name = "Opacity",
							min = 0, max = 1, step = 0.05,
							isPercent = true,
							get = function(info) return db.appearance.opacity end,
							set = function(info, value) 
								db.appearance.opacity = value
								nibSpellAlertConfig:UpdateAppearance()
							end,
							order = 10,
						},
					},
				},
			},
		};				
	end
	return options
end

function nibSpellAlertConfig:ChatCommand(input)
	InterfaceOptionsFrame_OpenToCategory(self.profilesFrame)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	InterfaceOptionsFrame:Raise()
end

function nibSpellAlertConfig:ConfigRefresh()
	db = self.db.profile;
end

function nibSpellAlertConfig:SetUpOptions()
	db = self.db.profile;

	LibStub("AceConfig-3.0"):RegisterOptionsTable("nibSpellAlertConfig", GetOptions);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("nibSpellAlertConfig-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db));
	
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("nibSpellAlertConfig", "nibSpellAlertConfig");
	self.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("nibSpellAlertConfig-Profiles", "Profiles", "nibSpellAlertConfig");
	
	self:RegisterChatCommand("nibspellalertconfig", "ChatCommand");
	self:RegisterChatCommand("nibspac", "ChatCommand");
end