local nibSpellAlertConfig = LibStub("AceAddon-3.0"):NewAddon("nibSpellAlertConfig", "AceConsole-3.0", "AceEvent-3.0");
local db;

local defaults = {
	profile = {
		configguide = {
			enabled = false,
			spells = {
				sides = "Clearcasting",
				top = "Shooting Stars",
				topleft = "Eclipse (Lunar)",
				topright = "Eclipse (Solar)",
			},
		},
		position = {
			x = 0,
			y = 0,
		},
		framelevel = {
			strata = "MEDIUM",
			level = 1,
		},		
		size = {
			scale = 1,
			width = 250,
			height = 250,
		},
		appearance = {
			opacity = 1,
		},
	},
};

local previewarea = {
	areas = {
		center = nil,
		outside = nil,
	},
	lines = {
		x = nil,
		y = nil,
		left = nil,
		right = nil,
		top = nil,
		bottom = nil,
	},
};

local SAScale = 0.8;
local LongWidth, ShortWidth = 256, 128;

local ClassAlerts_Sides = {
	["None"] = {
		id = 0,
		texture = "",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Death Knight
	["Killing Machine"] = {
		id = 51124,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\KILLING_MACHINE.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Sudden Doom"] = {
		id = 81340,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DOOM.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Druid
	["Clearcasting"] = {
		id = 16870,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\NATURES_GRACE.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Hunter
	["Focus Fire"] = {
		id = 88843,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\FOCUS_FIRE.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Mage
	["Arcane Missiles"] = {
		id = 5143,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\ARCANE_MISSILES.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Brain Freeze"] = {
		id = 57761,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\BRAIN_FREEZE.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Hot Streak"] = {
		id = 48108,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\HOT_STREAK.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Paladin
	["Art of War"] = {
		id = 59578,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\ART_OF_WAR.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Daybreak"] = {
		id = 88819,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\DAYBREAK.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Grand Crusader"] = {
		id = 85416,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\GRAND_CRUSADER.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Priest
	["Surge of Light"] = {
		id = 33151,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SURGE_OF_LIGHT.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Warlock
	["Nightfall"] = {
		id = 17941,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\NIGHTFALL.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Empowered Imp"] = {
		id = 47283,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\IMP_EMPOWERMENT.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Molten Core"] = {
		id = 71165,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\MOLTEN_CORE.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Warrior
	["Bloodsurge"] = {
		id = 46916,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\BLOOD_SURGE.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Sudden Death"] = {
		id = 52437,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DEATH.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Sword and Board"] = {
		id = 50227,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SWORD_AND_BOARD.BLP",
		positions = "Left + Right (Flipped)",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
};

local ClassAlerts_Top = {
	["None"] = {
		id = 0,
		texture = "",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Death Knight
	["Dark Transformation"] = {
		id = 93426,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\DARK_TRANSFORMATION.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Freezing Fog"] = {
		id = 59052,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\RIME.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Druid
	["Fury of Stormrage"] = {
		id = 81093,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\FURY_OF_STORMRAGE.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Shooting Stars"] = {
		id = 93400,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SHOOTING_STARS.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Hunter
	["Lock and Load"] = {
		id = 56453,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\LOCK_AND_LOAD.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Ready, Set, Aim"] = {
		id = 82926,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\MASTER_MARKSMAN.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Mage
	["Fingers of Frost"] = {
		id = 44544,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\FROZEN_FINGERS.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	["Impact"] = {
		id = 64343,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\IMPACT.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Paladin
	["Hand of Light"] = {
		id = 90174,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\HAND_OF_LIGHT.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Shaman
	["Maelstrom Weapon"] = {
		id = 60349,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\MAELSTROM_WEAPON.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Warlock
	["Backlash"] = {
		id = 34936,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\BACKLASH.BLP",
		positions = "Top",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
};

local ClassAlerts_TopRight = {
	["None"] = {
		id = 0,
		texture = "",
		positions = "TopRight",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Druid
	["Eclipse (Solar)"] = {
		id = 93430,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\ECLIPSE_SUN.BLP",
		positions = "TopRight",
		scale = 1,
		r = 244, g = 244, b = 244,
	},
};

local ClassAlerts_TopLeft = {
	["None"] = {
		id = 0,
		texture = "",
		positions = "TopLeft",
		scale = 1,
		r = 255, g = 255, b = 255,
	},
	-- Druid
	["Eclipse (Lunar)"] = {
		id = 93431,
		texture = "TEXTURES\\SPELLACTIVATIONOVERLAYS\\ECLIPSE_MOON.BLP",
		positions = "TopLeft",
		scale = 1,
		r = 244, g = 244, b = 244,
	},
};

-- Preview Area
-- Spell Alert refresh timer
local Timer_SendAlert = CreateFrame("FRAME")
local Timer_SendAlert_Int = 0;
Timer_SendAlert:Hide();
Timer_SendAlert:SetScript("OnUpdate", function(self, elapsed)
	Timer_SendAlert_Int = Timer_SendAlert_Int - elapsed;
	if (Timer_SendAlert_Int <= 0) then
		nibSpellAlertConfig:ShowPreviewAlerts("Sides");
		nibSpellAlertConfig:ShowPreviewAlerts("Top");		
		Timer_SendAlert_Int = 8;
	end
end);

-- Hide/Show Spell Alerts for Config Guide
function nibSpellAlertConfig:ShowPreviewAlerts(pos)
	local spellpos = {
		["Sides"] = ClassAlerts_Sides[db.configguide.spells.sides],
		["Top"] = ClassAlerts_Top[db.configguide.spells.top],
		["TopRight"] = ClassAlerts_TopRight[db.configguide.spells.topright],
		["TopLeft"] = ClassAlerts_TopLeft[db.configguide.spells.topleft],
	};
	SpellActivationOverlay_ShowAllOverlays(SpellActivationOverlayFrame, spellpos[pos].id, spellpos[pos].texture, spellpos[pos].positions, spellpos[pos].scale, spellpos[pos].r, spellpos[pos].g, spellpos[pos].b);
end

function nibSpellAlertConfig:HidePreviewAlerts(pos)
	local spellpos = {
		["Sides"] = ClassAlerts_Sides[db.configguide.spells.sides],
		["Top"] = ClassAlerts_Top[db.configguide.spells.top],
		["TopRight"] = ClassAlerts_TopRight[db.configguide.spells.topright],
		["TopLeft"] = ClassAlerts_TopLeft[db.configguide.spells.topleft],
	};
	SpellActivationOverlay_HideOverlays(SpellActivationOverlayFrame, spellpos[pos].id);
	Timer_SendAlert_Int = 0; -- Instantly refresh Alerts if in Config Guide
end

-- Toggle Config Guide
function nibSpellAlertConfig:ShowPreviewArea()
	if db.configguide.enabled then 
		-- Show Config Guide
		previewarea.areas.center:Show();
		-- previewarea.areas.outside:Show();
		
		nibSpellAlertConfig:ShowPreviewAlerts("Sides");
		nibSpellAlertConfig:ShowPreviewAlerts("Top");
		nibSpellAlertConfig:ShowPreviewAlerts("TopRight");
		nibSpellAlertConfig:ShowPreviewAlerts("TopLeft");
		
		Timer_SendAlert:Show();
	else
		-- Hide Config Guide
		Timer_SendAlert:Hide();
		
		previewarea.areas.center:Hide();
		-- previewarea.areas.outside:Hide();
		
		nibSpellAlertConfig:HidePreviewAlerts("Sides");
		nibSpellAlertConfig:HidePreviewAlerts("Top");
		nibSpellAlertConfig:HidePreviewAlerts("TopRight");
		nibSpellAlertConfig:HidePreviewAlerts("TopLeft");
	end
end

-- Position/Size
function nibSpellAlertConfig:UpdatePosition()
	-- Set standard Widths and Heights
	local swidth, sheight, scale = db.size.width, db.size.height, db.size.scale;
	local mwidth, mheight = swidth + (ShortWidth * SAScale), sheight + (ShortWidth * SAScale);
	local lwidth, lheight = swidth + (LongWidth * SAScale), sheight + (LongWidth * SAScale);
	local hwidth, hheight = (swidth / 2) + (64 * SAScale), (sheight / 2) + (64 * SAScale);

	-- Spell Alert frame
	SpellActivationOverlayFrame:ClearAllPoints();
	SpellActivationOverlayFrame:SetPoint("CENTER", UIParent, "CENTER", db.position.x, db.position.y);
	
	SpellActivationOverlayFrame:SetWidth(swidth);
	SpellActivationOverlayFrame:SetHeight(sheight);	
	SpellActivationOverlayFrame:SetScale(scale);
	
	SpellActivationOverlayFrame:SetFrameStrata(db.framelevel.strata);
	SpellActivationOverlayFrame:SetFrameLevel(db.framelevel.level);	
	
	-- Preview Area	
	previewarea.areas.center:ClearAllPoints();
	previewarea.areas.center:SetPoint("CENTER", UIParent, "CENTER", db.position.x, db.position.y);	
	previewarea.areas.center:SetWidth(swidth);
	previewarea.areas.center:SetHeight(sheight);	
	previewarea.areas.center:SetScale(db.size.scale);	
	previewarea.areas.center:SetFrameStrata(db.framelevel.strata);
	previewarea.areas.center:SetFrameLevel(db.framelevel.level);
	
	previewarea.areas.outside:ClearAllPoints();
	previewarea.areas.outside:SetPoint("CENTER", UIParent, "CENTER", db.position.x, db.position.y);	
	previewarea.areas.outside:SetWidth(lwidth);
	previewarea.areas.outside:SetHeight(lheight);	
	previewarea.areas.outside:SetScale(db.size.scale);	
	previewarea.areas.outside:SetFrameStrata(db.framelevel.strata);
	previewarea.areas.outside:SetFrameLevel(db.framelevel.level - 1);
	
	previewarea.lines.x:SetWidth(lwidth);
	previewarea.lines.y:SetHeight(lheight);
	
	previewarea.lines.left:SetPoint("CENTER", previewarea.areas.center, "CENTER", -hwidth, 0);
	previewarea.lines.left:SetHeight(mheight);
	
	previewarea.lines.right:SetPoint("CENTER", previewarea.areas.center, "CENTER", hwidth, 0);
	previewarea.lines.right:SetHeight(mheight);
	
	previewarea.lines.top:SetPoint("CENTER", previewarea.areas.center, "CENTER", 0, hheight);
	previewarea.lines.top:SetWidth(mwidth);
	
	previewarea.lines.bottom:SetPoint("CENTER", previewarea.areas.center, "CENTER", 0, -hheight);
	previewarea.lines.bottom:SetWidth(mwidth);
end

-- Appearance
function nibSpellAlertConfig:UpdateAppearance()
	SetCVar("spellActivationOverlayOpacity", db.appearance.opacity);
	SpellActivationOverlayFrame:SetAlpha(db.appearance.opacity);
end

function nibSpellAlertConfig:ProfChange()
	db = self.db.profile;
	nibSpellAlertConfig:ConfigRefresh();
	nibSpellAlertConfig:Refresh();
end

function nibSpellAlertConfig:Refresh()
	nibSpellAlertConfig:UpdatePosition();
	nibSpellAlertConfig:UpdateAppearance();
end

function nibSpellAlertConfig:PLAYER_LOGIN()
	-- Disable Config Guide
	db.configguide.enabled = false;
	db.configguide.spells = {sides = "Clearcasting", top = "Shooting Stars", topright = "Eclipse (Solar)", topleft = "Eclipse (Lunar)"};
	
	-- hooksecurefunc("SpellActivationOverlay_ShowOverlay", function(self, spellID, texturePath, position, scale, r, g, b, vFlip, hFlip)
		-- print(spellID); print(texturePath); print(position); print(scale); print(r); print(g); print(b);
	-- end);
	
	-- Refresh Addon
	nibSpellAlertConfig:Refresh();
end

-- Create Frames that make up the Config Guide
local function CreatePreviewArea()
	-- Areas
	previewarea.areas.center = CreateFrame("Frame", nil, UIParent);
	previewarea.areas.center:SetBackdrop({bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=], edgeFile = "", edgeSize = 0, tile = false, tileSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0}});
	previewarea.areas.center:SetBackdropColor(0, 1, 0, 0.4);
	previewarea.areas.center:Hide();
	
	previewarea.areas.outside = CreateFrame("Frame", nil, UIParent);
	previewarea.areas.outside:SetBackdrop({bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=], edgeFile = "", edgeSize = 0, tile = false, tileSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0}});
	previewarea.areas.outside:SetBackdropColor(0, 0, 1, 0.1);
	previewarea.areas.outside:Hide();
	
	-- Lines
	local function CreateNewLine()
		local templine;
		templine = CreateFrame("Frame", nil, UIParent);
		templine:SetParent(previewarea.areas.center);
		templine:SetPoint("CENTER", previewarea.areas.center, "CENTER", 0, 0);
		templine:SetBackdrop({bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=], edgeFile = "", edgeSize = 0, tile = false, tileSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0}});
		templine:SetBackdropColor(0.8, 0.8, 0.8, 0.6);
		templine:SetHeight(2);
		templine:SetWidth(2);
		return templine;
	end
	
	previewarea.lines.x = CreateNewLine();
	previewarea.lines.y = CreateNewLine();
	previewarea.lines.left = CreateNewLine();
	previewarea.lines.right = CreateNewLine();
	previewarea.lines.top = CreateNewLine();
	previewarea.lines.bottom = CreateNewLine();
end

function nibSpellAlertConfig:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("nibSpellAlertConfigDB", defaults, "Default");
		
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfChange");
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfChange");
	self.db.RegisterCallback(self, "OnProfileReset", "ProfChange");
	
	nibSpellAlertConfig:SetUpOptions();	
	db = self.db.profile;
	
	CreatePreviewArea();
	
	self:RegisterEvent("PLAYER_LOGIN");
end